"""
Unit tests for the stem.response.getinfo.GetInfoResponse class.
"""

import unittest

import stem.response
import stem.response.getinfo
import stem.socket
import stem.util.str_tools

from stem.response import ControlMessage

SINGLE_RESPONSE = """\
250-version=0.2.3.11-alpha-dev
250 OK"""

BATCH_RESPONSE = """\
250-version=0.2.3.11-alpha-dev
250-address=67.137.76.214
250-fingerprint=5FDE0422045DF0E1879A3738D09099EB4A0C5BA0
250 OK"""

MULTILINE_RESPONSE = """\
250-version=0.2.3.11-alpha-dev (git-ef0bc7f8f26a917c)
250+config-text=
ControlPort 9051
DataDirectory /home/atagar/.tor
ExitPolicy reject *:*
Log notice stdout
Nickname Unnamed
ORPort 9050
.
250 OK"""

NON_KEY_VALUE_ENTRY = """\
250-version=0.2.3.11-alpha-dev
250-address 67.137.76.214
250 OK"""

UNRECOGNIZED_KEY_ENTRY = """\
552 Unrecognized key "blackhole"
"""

MISSING_MULTILINE_NEWLINE = """\
250+config-text=ControlPort 9051
DataDirectory /home/atagar/.tor
.
250 OK"""


class TestGetInfoResponse(unittest.TestCase):
  def test_empty_response(self):
    """
    Parses a GETINFO reply without options (just calling "GETINFO").
    """

    control_message = ControlMessage.from_str('250 OK\r\n', 'GETINFO')

    # now this should be a GetInfoResponse (ControlMessage subclass)
    self.assertTrue(isinstance(control_message, stem.response.ControlMessage))
    self.assertTrue(isinstance(control_message, stem.response.getinfo.GetInfoResponse))

    self.assertEqual({}, control_message.entries)

  def test_single_response(self):
    """
    Parses a GETINFO reply response for a single parameter.
    """

    control_message = ControlMessage.from_str(SINGLE_RESPONSE, 'GETINFO', normalize = True)
    self.assertEqual({'version': b'0.2.3.11-alpha-dev'}, control_message.entries)

  def test_batch_response(self):
    """
    Parses a GETINFO reply for muiltiple parameters.
    """

    expected = {
      'version': b'0.2.3.11-alpha-dev',
      'address': b'67.137.76.214',
      'fingerprint': b'5FDE0422045DF0E1879A3738D09099EB4A0C5BA0',
    }

    control_message = ControlMessage.from_str(BATCH_RESPONSE, 'GETINFO', normalize = True)
    self.assertEqual(expected, control_message.entries)

  def test_multiline_response(self):
    """
    Parses a GETINFO reply for multiple parameters including a multi-line
    value.
    """

    expected = {
      'version': b'0.2.3.11-alpha-dev (git-ef0bc7f8f26a917c)',
      'config-text': b'\n'.join(stem.util.str_tools._to_bytes(MULTILINE_RESPONSE).splitlines()[2:8]),
    }

    control_message = ControlMessage.from_str(MULTILINE_RESPONSE, 'GETINFO', normalize = True)
    self.assertEqual(expected, control_message.entries)

  def test_invalid_non_mapping_content(self):
    """
    Parses a malformed GETINFO reply containing a line that isn't a key=value
    entry.
    """

    control_message = ControlMessage.from_str(NON_KEY_VALUE_ENTRY, normalize = True)
    self.assertRaises(stem.ProtocolError, stem.response.convert, 'GETINFO', control_message)

  def test_unrecognized_key_response(self):
    """
    Parses a GETCONF reply that contains an error code with an unrecognized key.
    """

    try:
      control_message = ControlMessage.from_str(UNRECOGNIZED_KEY_ENTRY, normalize = True)
      stem.response.convert('GETINFO', control_message)
      self.fail('expected a stem.InvalidArguments to be raised')
    except stem.InvalidArguments as exc:
      self.assertEqual(exc.arguments, ['blackhole'])

  def test_invalid_multiline_content(self):
    """
    Parses a malformed GETINFO reply with a multi-line entry missing a newline
    between its key and value. This is a proper controller message, but
    malformed according to the GETINFO's spec.
    """

    control_message = ControlMessage.from_str(MISSING_MULTILINE_NEWLINE, normalize = True)
    self.assertRaises(stem.ProtocolError, stem.response.convert, 'GETINFO', control_message)



#!/usr/bin/env python
# Copyright 2016-2019, Damian Johnson and The Tor Project
# See LICENSE for licensing information

"""
Caches tor's latest fallback directories.
"""

import re
import sys

import stem.directory
import stem.util.system

try:
  # account for urllib's change between python 2.x and 3.x
  import urllib.request as urllib
except ImportError:
  import urllib2 as urllib

GITWEB_FALLBACK_LOG = 'https://gitweb.torproject.org/tor.git/log/src/app/config/fallback_dirs.inc'
FALLBACK_DIR_LINK = "href='/tor.git/commit/src/app/config/fallback_dirs.inc\\?id=([^']*)'"

if __name__ == '__main__':
  try:
    fallback_dir_page = urllib.urlopen(GITWEB_FALLBACK_LOG).read()
    fallback_dir_commit = re.search(FALLBACK_DIR_LINK, fallback_dir_page).group(1)
  except:
    print("Unable to determine the latest commit to edit tor's fallback directories: %s" % sys.exc_info()[1])
    sys.exit(1)

  try:
    stem_commit = stem.util.system.call('git rev-parse HEAD')[0]
  except IOError as exc:
    print("Unable to determine stem's current commit: %s" % exc)
    sys.exit(1)

  print('Latest tor commit editing fallback directories: %s' % fallback_dir_commit)
  print('Current stem commit: %s' % stem_commit)
  print('')

  cached_fallback_directories = stem.directory.Fallback.from_cache()
  latest_fallback_directories = stem.directory.Fallback.from_remote()

  if cached_fallback_directories == latest_fallback_directories:
    print('Fallback directories are already up to date, nothing to do.')
    sys.exit(0)

  # all fallbacks have the same header metadata, so just picking one

  headers = latest_fallback_directories.values()[0].header if latest_fallback_directories else None

  print('Differences detected...\n')
  print(stem.directory._fallback_directory_differences(cached_fallback_directories, latest_fallback_directories))
  stem.directory.Fallback._write(latest_fallback_directories, fallback_dir_commit, stem_commit, headers)



from stem.control import Controller

with Controller.from_port(port = 9051) as controller:
  controller.authenticate()
  desc = controller.get_hidden_service_descriptor('3g2upl4pq6kufc4m')

  print("DuckDuckGo's introduction points are...\n")

  for introduction_point in desc.introduction_points():
    print('  %s:%s => %s' % (introduction_point.address, introduction_point.port, introduction_point.identifier))



"""
Unit tests for the stem.util.term functions.
"""

import unittest

import stem.util.term

from stem.util.term import Color, Attr


class TestTerminal(unittest.TestCase):
  def test_encoding(self):
    """
    Exercises our encoding function.
    """

    self.assertEqual(None, stem.util.term.encoding())
    self.assertEqual('\x1b[31m', stem.util.term.encoding(Color.RED))
    self.assertEqual('\x1b[31;1m', stem.util.term.encoding(Color.RED, Attr.BOLD))

  def test_format(self):
    """
    Exercises our format function.
    """

    self.assertEqual('hi!', stem.util.term.format('hi!'))
    self.assertEqual('\x1b[31mhi!\x1b[0m', stem.util.term.format('hi!', Color.RED))
    self.assertEqual('\x1b[31;1mhi!\x1b[0m', stem.util.term.format('hi!', Color.RED, Attr.BOLD))
    self.assertEqual('\x1b[31mhi\nthere!\x1b[0m', stem.util.term.format('hi\nthere!', Color.RED))
    self.assertEqual('\x1b[31mhi\x1b[0m\n\x1b[31mthere!\x1b[0m', stem.util.term.format('hi\nthere!', Color.RED, Attr.LINES))
    self.assertEqual('\001\x1b[31m\002hi!\001\x1b[0m\002', stem.util.term.format('hi!', Color.RED, Attr.READLINE_ESCAPE))



"""
Unit tests for the DirectoryAuthority of stem.descriptor.networkstatus.
"""

import unittest

import test.require

from stem.descriptor.networkstatus import (
  DirectoryAuthority,
  KeyCertificate,
)

DIR_SOURCE_LINE = 'turtles 27B6B5996C426270A5C95488AA5BCEB6BCC86956 no.place.com 76.73.17.194 9030 9090'


class TestDirectoryAuthority(unittest.TestCase):
  def test_minimal_consensus_authority(self):
    """
    Parses a minimal directory authority for a consensus.
    """

    authority = DirectoryAuthority.create()

    self.assertTrue(authority.nickname.startswith('Unnamed'))
    self.assertEqual(40, len(authority.fingerprint))
    self.assertEqual('no.place.com', authority.hostname)
    self.assertEqual(9030, authority.dir_port)
    self.assertEqual(9090, authority.or_port)
    self.assertEqual(False, authority.is_legacy)
    self.assertEqual('Mike Perry <email>', authority.contact)
    self.assertEqual(40, len(authority.vote_digest))
    self.assertEqual(None, authority.legacy_dir_key)
    self.assertEqual(None, authority.key_certificate)
    self.assertEqual([], authority.get_unrecognized_lines())

  def test_minimal_vote_authority(self):
    """
    Parses a minimal directory authority for a vote.
    """

    authority = DirectoryAuthority.create(is_vote = True)

    self.assertTrue(authority.nickname.startswith('Unnamed'))
    self.assertEqual(40, len(authority.fingerprint))
    self.assertEqual('no.place.com', authority.hostname)
    self.assertEqual(9030, authority.dir_port)
    self.assertEqual(9090, authority.or_port)
    self.assertEqual(False, authority.is_legacy)
    self.assertEqual('Mike Perry <email>', authority.contact)
    self.assertEqual(None, authority.vote_digest)
    self.assertEqual(None, authority.legacy_dir_key)
    self.assertEqual([], authority.get_unrecognized_lines())

  @test.require.cryptography
  def test_descriptor_signing(self):
    self.assertRaisesWith(NotImplementedError, 'Signing of DirectoryAuthority not implemented', DirectoryAuthority.create, sign = True)

  def test_unrecognized_line(self):
    """
    Includes unrecognized content in the descriptor.
    """

    authority = DirectoryAuthority.create({'pepperjack': 'is oh so tasty!'})
    self.assertEqual(['pepperjack is oh so tasty!'], authority.get_unrecognized_lines())

  def test_legacy_authority(self):
    """
    Parses an authority using the '-legacy' format.
    """

    content = 'dir-source gabelmoo-legacy 81349FC1F2DBA2C2C11B45CB9706637D480AB913 131.188.40.189 131.188.40.189 80 443'
    authority = DirectoryAuthority(content, is_vote = False)

    self.assertEqual('gabelmoo-legacy', authority.nickname)
    self.assertEqual('81349FC1F2DBA2C2C11B45CB9706637D480AB913', authority.fingerprint)
    self.assertEqual('131.188.40.189', authority.hostname)
    self.assertEqual('131.188.40.189', authority.address)
    self.assertEqual(80, authority.dir_port)
    self.assertEqual(443, authority.or_port)
    self.assertEqual(True, authority.is_legacy)
    self.assertEqual(None, authority.contact)
    self.assertEqual(None, authority.vote_digest)
    self.assertEqual(None, authority.legacy_dir_key)
    self.assertEqual(None, authority.key_certificate)
    self.assertEqual([], authority.get_unrecognized_lines())

  def test_first_line(self):
    """
    Includes a non-mandatory field before the 'dir-source' line.
    """

    content = b'ho-hum 567\n' + DirectoryAuthority.content()
    self.assertRaises(ValueError, DirectoryAuthority, content, True)

    authority = DirectoryAuthority(content, False)
    self.assertTrue(authority.nickname.startswith('Unnamed'))
    self.assertEqual(['ho-hum 567'], authority.get_unrecognized_lines())

  def test_missing_fields(self):
    """
    Parse an authority where a mandatory field is missing.
    """

    for excluded_field in ('dir-source', 'contact'):
      content = DirectoryAuthority.content(exclude = (excluded_field,))
      self.assertRaises(ValueError, DirectoryAuthority, content, True)

      authority = DirectoryAuthority(content, False)

      if excluded_field == 'dir-source':
        self.assertEqual('Mike Perry <email>', authority.contact)
      else:
        self.assertTrue(authority.nickname.startswith('Unnamed'))

  def test_blank_lines(self):
    """
    Includes blank lines, which should be ignored.
    """

    authority = DirectoryAuthority.create({'dir-source': DIR_SOURCE_LINE + '\n\n\n'})
    self.assertEqual('Mike Perry <email>', authority.contact)

  def test_duplicate_lines(self):
    """
    Duplicates linesin the entry.
    """

    lines = DirectoryAuthority.content().split(b'\n')

    for index, duplicate_line in enumerate(lines):
      content = b'\n'.join(lines[:index] + [duplicate_line] + lines[index:])
      self.assertRaises(ValueError, DirectoryAuthority, content, True)

      authority = DirectoryAuthority(content, False)
      self.assertTrue(authority.nickname.startswith('Unnamed'))

  def test_missing_dir_source_field(self):
    """
    Excludes fields from the 'dir-source' line.
    """

    for missing_value in DIR_SOURCE_LINE.split(' '):
      dir_source = DIR_SOURCE_LINE.replace(missing_value, '').replace('  ', ' ')
      content = DirectoryAuthority.content({'dir-source': dir_source})
      self.assertRaises(ValueError, DirectoryAuthority, content, True)

      authority = DirectoryAuthority(content, False)

      self.assertEqual(None, authority.nickname)
      self.assertEqual(None, authority.fingerprint)
      self.assertEqual(None, authority.hostname)
      self.assertEqual(None, authority.address)
      self.assertEqual(None, authority.dir_port)
      self.assertEqual(None, authority.or_port)

  def test_malformed_fingerprint(self):
    """
    Includes a malformed fingerprint on the 'dir-source' line.
    """

    test_values = (
      '',
      'zzzzz',
      'zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz',
    )

    for value in test_values:
      dir_source = DIR_SOURCE_LINE.replace('27B6B5996C426270A5C95488AA5BCEB6BCC86956', value)
      content = DirectoryAuthority.content({'dir-source': dir_source})
      self.assertRaises(ValueError, DirectoryAuthority, content, True)

      authority = DirectoryAuthority(content, False)
      self.assertEqual(None, authority.fingerprint)

  def test_malformed_address(self):
    """
    Includes a malformed ip address on the 'dir-source' line.
    """

    test_values = (
      '',
      '71.35.150.',
      '71.35..29',
      '71.35.150',
      '71.35.150.256',
      '[fd9f:2e19:3bcf::02:9970]',
    )

    for value in test_values:
      dir_source = DIR_SOURCE_LINE.replace('76.73.17.194', value)
      content = DirectoryAuthority.content({'dir-source': dir_source})
      self.assertRaises(ValueError, DirectoryAuthority, content, True)

      authority = DirectoryAuthority(content, False)
      self.assertEqual(None, authority.address)

  def test_malformed_port(self):
    """
    Includes a malformed orport or dirport on the 'dir-source' line.
    """

    test_values = (
      '',
      '-1',
      '399482',
      'blarg',
    )

    for value in test_values:
      for include_or_port in (False, True):
        for include_dir_port in (False, True):
          if not include_or_port and not include_dir_port:
            continue

          dir_source = DIR_SOURCE_LINE

          if include_or_port:
            dir_source = dir_source.replace('9090', value)

          if include_dir_port:
            dir_source = dir_source.replace('9030', value)

          content = DirectoryAuthority.content({'dir-source': dir_source})
          self.assertRaises(ValueError, DirectoryAuthority, content, True)

          authority = DirectoryAuthority(content, False)

          actual_value = authority.or_port if include_or_port else authority.dir_port
          self.assertEqual(None, actual_value)

  def test_legacy_dir_key(self):
    """
    Includes a 'legacy-dir-key' line with both valid and invalid content.
    """

    test_value = '65968CCB6BECB5AA88459C5A072624C6995B6B72'
    authority = DirectoryAuthority.create({'legacy-dir-key': test_value}, is_vote = True)
    self.assertEqual(test_value, authority.legacy_dir_key)

    # check that we'll fail if legacy-dir-key appears in a consensus
    content = DirectoryAuthority.content({'legacy-dir-key': test_value})
    self.assertRaises(ValueError, DirectoryAuthority, content, True)

    test_values = (
      '',
      'zzzzz',
      'zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz',
    )

    for value in test_values:
      content = DirectoryAuthority.content({'legacy-dir-key': value})
      self.assertRaises(ValueError, DirectoryAuthority, content, True)

      authority = DirectoryAuthority(content, False)
      self.assertEqual(None, authority.legacy_dir_key)

  def test_key_certificate(self):
    """
    Includes or exclude a key certificate from the directory entry.
    """

    key_cert = KeyCertificate.content()

    # include a key cert with a consensus
    content = DirectoryAuthority.content() + b'\n' + key_cert
    self.assertRaises(ValueError, DirectoryAuthority, content, True)

    authority = DirectoryAuthority(content, False)
    self.assertTrue(authority.nickname.startswith('Unnamed'))

    # exclude  key cert from a vote

    content = b'\n'.join(DirectoryAuthority.content(is_vote = True).splitlines()[:-5])
    self.assertRaises(ValueError, DirectoryAuthority, content, True, True)

    authority = DirectoryAuthority(content, False, True)
    self.assertTrue(authority.nickname.startswith('Unnamed'))



"""
Unit tests for the stem.response.protocolinfo.ProtocolInfoResponse class.
"""

import unittest

import stem.response
import stem.response.protocolinfo
import stem.socket
import stem.util.proc
import stem.util.system
import stem.version

from stem.response import ControlMessage
from stem.response.protocolinfo import AuthMethod

try:
  # added in python 3.3
  from unittest.mock import Mock, patch
except ImportError:
  from mock import Mock, patch

NO_AUTH = """250-PROTOCOLINFO 1
250-AUTH METHODS=NULL
250-VERSION Tor="0.2.1.30"
250 OK"""

PASSWORD_AUTH = """250-PROTOCOLINFO 1
250-AUTH METHODS=HASHEDPASSWORD
250-VERSION Tor="0.2.1.30"
250 OK"""

COOKIE_AUTH = r"""250-PROTOCOLINFO 1
250-AUTH METHODS=COOKIE COOKIEFILE="/tmp/my data\\\"dir//control_auth_cookie"
250-VERSION Tor="0.2.1.30"
250 OK"""

MULTIPLE_AUTH = """250-PROTOCOLINFO 1
250-AUTH METHODS=COOKIE,HASHEDPASSWORD COOKIEFILE="/home/atagar/.tor/control_auth_cookie"
250-VERSION Tor="0.2.1.30"
250 OK"""

UNKNOWN_AUTH = """250-PROTOCOLINFO 1
250-AUTH METHODS=MAGIC,HASHEDPASSWORD,PIXIE_DUST
250-VERSION Tor="0.2.1.30"
250 OK"""

MINIMUM_RESPONSE = """250-PROTOCOLINFO 5
250 OK"""

UNICODE_COOKIE_PATH = r"""250-PROTOCOLINFO 1
250-AUTH METHODS=COOKIE COOKIEFILE="/home/user/\346\226\207\346\241\243/tor-browser_en-US/Browser/TorBrowser/Data/Tor/control_auth_cookie"
250-VERSION Tor="0.2.1.30"
250 OK"""

RELATIVE_COOKIE_PATH = r"""250-PROTOCOLINFO 1
250-AUTH METHODS=COOKIE COOKIEFILE="./tor-browser_en-US/Data/control_auth_cookie"
250-VERSION Tor="0.2.1.30"
250 OK"""

EXPECTED_UNICODE_PATH = b"/home/user/\346\226\207\346\241\243/tor-browser_en-US/Browser/TorBrowser/Data/Tor/control_auth_cookie".decode('utf-8')


class TestProtocolInfoResponse(unittest.TestCase):
  def test_convert(self):
    """
    Exercises functionality of the convert method both when it works and
    there's an error.
    """

    # working case
    control_message = ControlMessage.from_str(NO_AUTH, 'PROTOCOLINFO', normalize = True)

    # now this should be a ProtocolInfoResponse (ControlMessage subclass)
    self.assertTrue(isinstance(control_message, stem.response.ControlMessage))
    self.assertTrue(isinstance(control_message, stem.response.protocolinfo.ProtocolInfoResponse))

    # exercise some of the ControlMessage functionality
    raw_content = (NO_AUTH + '\n').replace('\n', '\r\n')
    self.assertEqual(raw_content, control_message.raw_content())
    self.assertTrue(str(control_message).startswith('PROTOCOLINFO 1'))

    # attempt to convert the wrong type
    self.assertRaises(TypeError, stem.response.convert, 'PROTOCOLINFO', 'hello world')

    # attempt to convert a different message type
    self.assertRaises(stem.ProtocolError, ControlMessage.from_str, '650 BW 32326 2856\r\n', 'PROTOCOLINFO')

  def test_no_auth(self):
    """
    Checks a response when there's no authentication.
    """

    control_message = ControlMessage.from_str(NO_AUTH, 'PROTOCOLINFO', normalize = True)
    self.assertEqual(1, control_message.protocol_version)
    self.assertEqual(stem.version.Version('0.2.1.30'), control_message.tor_version)
    self.assertEqual((AuthMethod.NONE, ), control_message.auth_methods)
    self.assertEqual((), control_message.unknown_auth_methods)
    self.assertEqual(None, control_message.cookie_path)

  def test_password_auth(self):
    """
    Checks a response with password authentication.
    """

    control_message = ControlMessage.from_str(PASSWORD_AUTH, 'PROTOCOLINFO', normalize = True)
    self.assertEqual((AuthMethod.PASSWORD, ), control_message.auth_methods)

  def test_cookie_auth(self):
    """
    Checks a response with cookie authentication and a path including escape
    characters.
    """

    control_message = ControlMessage.from_str(COOKIE_AUTH, 'PROTOCOLINFO', normalize = True)
    self.assertEqual((AuthMethod.COOKIE, ), control_message.auth_methods)
    self.assertEqual('/tmp/my data\\"dir//control_auth_cookie', control_message.cookie_path)

  def test_multiple_auth(self):
    """
    Checks a response with multiple authentication methods.
    """

    control_message = ControlMessage.from_str(MULTIPLE_AUTH, 'PROTOCOLINFO', normalize = True)
    self.assertEqual((AuthMethod.COOKIE, AuthMethod.PASSWORD), control_message.auth_methods)
    self.assertEqual('/home/atagar/.tor/control_auth_cookie', control_message.cookie_path)

  def test_unknown_auth(self):
    """
    Checks a response with an unrecognized authtentication method.
    """

    control_message = ControlMessage.from_str(UNKNOWN_AUTH, 'PROTOCOLINFO', normalize = True)
    self.assertEqual((AuthMethod.UNKNOWN, AuthMethod.PASSWORD), control_message.auth_methods)
    self.assertEqual(('MAGIC', 'PIXIE_DUST'), control_message.unknown_auth_methods)

  def test_minimum_response(self):
    """
    Checks a PROTOCOLINFO response that only contains the minimum amount of
    information to be a valid response.
    """

    control_message = ControlMessage.from_str(MINIMUM_RESPONSE, 'PROTOCOLINFO', normalize = True)
    self.assertEqual(5, control_message.protocol_version)
    self.assertEqual(None, control_message.tor_version)
    self.assertEqual((), control_message.auth_methods)
    self.assertEqual((), control_message.unknown_auth_methods)
    self.assertEqual(None, control_message.cookie_path)

  @patch('sys.getfilesystemencoding', Mock(return_value = 'UTF-8'))
  def test_unicode_cookie(self):
    """
    Checks an authentication cookie with a unicode path.
    """

    control_message = ControlMessage.from_str(UNICODE_COOKIE_PATH, 'PROTOCOLINFO', normalize = True)
    self.assertEqual(EXPECTED_UNICODE_PATH, control_message.cookie_path)



import StringIO
import time

import pycurl

import stem.control

# Static exit for us to make 2-hop circuits through. Picking aurora, a
# particularly beefy one...
#
#   https://metrics.torproject.org/rs.html#details/379FB450010D17078B3766C2273303C358C3A442

EXIT_FINGERPRINT = '379FB450010D17078B3766C2273303C358C3A442'

SOCKS_PORT = 9050
CONNECTION_TIMEOUT = 30  # timeout before we give up on a circuit

def query(url):
  """
  Uses pycurl to fetch a site using the proxy on the SOCKS_PORT.
  """

  output = StringIO.StringIO()

  query = pycurl.Curl()
  query.setopt(pycurl.URL, url)
  query.setopt(pycurl.PROXY, 'localhost')
  query.setopt(pycurl.PROXYPORT, SOCKS_PORT)
  query.setopt(pycurl.PROXYTYPE, pycurl.PROXYTYPE_SOCKS5_HOSTNAME)
  query.setopt(pycurl.CONNECTTIMEOUT, CONNECTION_TIMEOUT)
  query.setopt(pycurl.WRITEFUNCTION, output.write)

  try:
    query.perform()
    return output.getvalue()
  except pycurl.error as exc:
    raise ValueError("Unable to reach %s (%s)" % (url, exc))


def scan(controller, path):
  """
  Fetch check.torproject.org through the given path of relays, providing back
  the time it took.
  """

  circuit_id = controller.new_circuit(path, await_build = True)

  def attach_stream(stream):
    if stream.status == 'NEW':
      controller.attach_stream(stream.id, circuit_id)

  controller.add_event_listener(attach_stream, stem.control.EventType.STREAM)

  try:
    controller.set_conf('__LeaveStreamsUnattached', '1')  # leave stream management to us
    start_time = time.time()

    check_page = query('https://check.torproject.org/')

    if 'Congratulations. This browser is configured to use Tor.' not in check_page:
      raise ValueError("Request didn't have the right content")

    return time.time() - start_time
  finally:
    controller.remove_event_listener(attach_stream)
    controller.reset_conf('__LeaveStreamsUnattached')


with stem.control.Controller.from_port() as controller:
  controller.authenticate()

  relay_fingerprints = [desc.fingerprint for desc in controller.get_network_statuses()]

  for fingerprint in relay_fingerprints:
    try:
      time_taken = scan(controller, [fingerprint, EXIT_FINGERPRINT])
      print('%s => %0.2f seconds' % (fingerprint, time_taken))
    except Exception as exc:
      print('%s => %s' % (fingerprint, exc))



"""
Unit tests for the stem.util.tor_tools functions.
"""

import unittest

import stem.util.str_tools
import stem.util.tor_tools


class TestTorTools(unittest.TestCase):
  def test_is_valid_fingerprint(self):
    """
    Checks the is_valid_fingerprint function.
    """

    valid_fingerprints = (
      '$A7569A83B5706AB1B1A9CB52EFF7D2D32E4553EB',
      '$a7569a83b5706ab1b1a9cb52eff7d2d32e4553eb',
      stem.util.str_tools._to_bytes('$A7569A83B5706AB1B1A9CB52EFF7D2D32E4553EB'),
      stem.util.str_tools._to_unicode('$A7569A83B5706AB1B1A9CB52EFF7D2D32E4553EB'),
    )

    invalid_fingerprints = (
      None,
      '',
      5,
      ['A7569A83B5706AB1B1A9CB52EFF7D2D32E4553EB'],
      'A7569A83B5706AB1B1A9CB52EFF7D2D32E4553EB',
      '$A7569A83B5706AB1B1A9CB52EFF7D2D32E4553E',
      '$A7569A83B5706AB1B1A9CB52EFF7D2D32E4553E33',
      '$A7569A83B5706AB1B1A9CB52EFF7D2D32E4553EG',
    )

    for fingerprint in valid_fingerprints:
      self.assertTrue(stem.util.tor_tools.is_valid_fingerprint(fingerprint, True))

    for fingerprint in invalid_fingerprints:
      self.assertFalse(stem.util.tor_tools.is_valid_fingerprint(fingerprint, True))

  def test_is_valid_nickname(self):
    """
    Checks the is_valid_nickname function.
    """

    valid_nicknames = (
      'caerSidi',
      'a',
      'abcABC123',
      stem.util.str_tools._to_bytes('caerSidi'),
      stem.util.str_tools._to_unicode('caerSidi'),
    )

    invalid_nicknames = (
      None,
      '',
      5,
      'toolongggggggggggggg',
      'bad_character',
    )

    for nickname in valid_nicknames:
      self.assertTrue(stem.util.tor_tools.is_valid_nickname(nickname))

    for nickname in invalid_nicknames:
      self.assertFalse(stem.util.tor_tools.is_valid_nickname(nickname))

  def test_is_valid_circuit_id(self):
    """
    Checks the is_valid_circuit_id function.
    """

    valid_circuit_ids = (
      '0',
      '2',
      'abcABC123',
      stem.util.str_tools._to_bytes('2'),
      stem.util.str_tools._to_unicode('2'),
    )

    invalid_circuit_ids = (
      None,
      '',
      0,
      2,
      'toolonggggggggggg',
      'bad_character',
    )

    for circuit_id in valid_circuit_ids:
      self.assertTrue(stem.util.tor_tools.is_valid_circuit_id(circuit_id))

    for circuit_id in invalid_circuit_ids:
      self.assertFalse(stem.util.tor_tools.is_valid_circuit_id(circuit_id))

  def test_is_valid_hex_digits(self):
    """
    Checks the is_valid_hex_digits function.
    """

    valid_hex_digits = (
      '12345',
      'AbCdE',
      stem.util.str_tools._to_bytes('AbCdE'),
      stem.util.str_tools._to_unicode('AbCdE'),
    )

    invalid_hex_digits = (
      None,
      '',
      5,
      'X',
      '1234',
      'ABCDEF',
      [1, '2', (3, 4)]
    )

    for hex_digits in valid_hex_digits:
      self.assertTrue(stem.util.tor_tools.is_hex_digits(hex_digits, 5))

    for hex_digits in invalid_hex_digits:
      self.assertFalse(stem.util.tor_tools.is_hex_digits(hex_digits, 5))



# Copyright 2011-2019, Damian Johnson and The Tor Project
# See LICENSE for licensing information

"""
Variety of filters for the python unit testing output, which can be chained
together for improved readability.
"""

import re
import threading
import traceback
import sys

import stem.util.enum
import stem.util.test_tools

from stem.util import system, term

COLOR_SUPPORT = sys.stdout.isatty() and not system.is_windows()

DIVIDER = '=' * 70
HEADER_ATTR = (term.Color.CYAN, term.Attr.BOLD)
CATEGORY_ATTR = (term.Color.GREEN, term.Attr.BOLD)

NO_NL = 'no newline'
STDERR = 'stderr'

# formatting for various categories of messages

STATUS = (term.Color.BLUE, term.Attr.BOLD)
SUBSTATUS = (term.Color.BLUE, )

SUCCESS = (term.Color.GREEN, term.Attr.BOLD)
ERROR = (term.Color.RED, term.Attr.BOLD)

LineType = stem.util.enum.Enum('OK', 'FAIL', 'ERROR', 'SKIPPED', 'CONTENT')

LINE_ENDINGS = {
  ' ... ok': LineType.OK,
  ' ... FAIL': LineType.FAIL,
  ' ... ERROR': LineType.ERROR,
  ' ... skipped': LineType.SKIPPED,
}

LINE_ATTR = {
  LineType.OK: (term.Color.GREEN,),
  LineType.FAIL: (term.Color.RED, term.Attr.BOLD),
  LineType.ERROR: (term.Color.RED, term.Attr.BOLD),
  LineType.SKIPPED: (term.Color.BLUE,),
  LineType.CONTENT: (term.Color.CYAN,),
}

SUPPRESS_STDOUT = False  # prevent anything from being printed to stdout


def println(msg = '', *attr):
  if SUPPRESS_STDOUT and STDERR not in attr:
    return

  attr = _flatten(attr)
  no_newline = False
  stream = sys.stderr if STDERR in attr else sys.stdout

  if NO_NL in attr:
    no_newline = True
    attr.remove(NO_NL)

  if STDERR in attr:
    attr.remove(STDERR)

  if COLOR_SUPPORT and attr:
    msg = term.format(msg, *attr)

  if not no_newline:
    msg += '\n'

  stream.write(msg)
  stream.flush()


def print_divider(msg, is_header = False):
  attr = HEADER_ATTR if is_header else CATEGORY_ATTR
  println('%s\n%s\n%s\n' % (DIVIDER, msg.center(70), DIVIDER), *attr)


def print_logging(logging_buffer):
  if SUPPRESS_STDOUT or logging_buffer is None:
    return

  if not logging_buffer.is_empty():
    for entry in logging_buffer:
      println(entry.replace('\n', '\n  '), term.Color.MAGENTA)

    print('')


def thread_stacktraces():
  """
  Provides a dump of the stacktrace information for all active threads.

  :returns: **dict** that maps thread names to their stacktrace
  """

  stacktraces = {}

  for thread in threading.enumerate():
    frame = sys._current_frames().get(thread.ident, None)

    if frame:
      stacktraces[thread.name] = ''.join(traceback.format_stack(frame))
    else:
      stacktraces[thread.name] = 'No traceback available'

  return stacktraces


def apply_filters(testing_output, *filters):
  """
  Gets the tests results, possibly processed through a series of filters. The
  filters are applied in order, each getting the output of the previous.

  A filter's input arguments should be the line's (type, content) and the
  output is either a string with the new content or None if the line should be
  omitted.

  :param str testing_output: output from the unit testing
  :param list filters: functors to be applied to each line of the results

  :returns: str with the processed test results
  """

  results = []

  for line in testing_output.splitlines():
    # determine the type of the line
    line_type = LineType.CONTENT

    for ending in LINE_ENDINGS:
      if ending in line:
        line_type = LINE_ENDINGS[ending]
        break

    for result_filter in filters:
      line = result_filter(line_type, line)

      if line is None:
        break

    if line is not None:
      results.append(line)

  return '\n'.join(results) + '\n'


def colorize(line_type, line_content):
  """
  Applies escape sequences so each line is colored according to its type.
  """

  if COLOR_SUPPORT:
    line_content = term.format(line_content, *LINE_ATTR[line_type])

  return line_content


def strip_module(line_type, line_content):
  """
  Removes the module name from testing output. This information tends to be
  repetitive, and redundant with the headers.
  """

  m = re.match('.*( \\(test\\..*?\\)).*', line_content)

  if m:
    line_content = line_content.replace(m.groups()[0], '', 1)

  return line_content


def runtimes(line_type, line_content):
  """
  Provides test runtimes if showing verbose results.
  """

  m = re.search('(test\\.[^)]*)', line_content)

  if m and line_type == LineType.OK:
    test = '%s.%s' % (m.group(0), line_content.split()[0])
    runtime = stem.util.test_tools.test_runtimes().get(test)

    if runtime is None:
      pass
    if runtime >= 1.0:
      line_content = '%s (%0.2fs)' % (line_content, runtime)
    else:
      line_content = '%s (%i ms)' % (line_content, runtime * 1000)

  return line_content


def align_results(line_type, line_content):
  """
  Strips the normal test results, and adds a right aligned variant instead with
  a bold attribute.
  """

  if line_type == LineType.CONTENT:
    return line_content

  # strip our current ending
  for ending in LINE_ENDINGS:
    if LINE_ENDINGS[ending] == line_type:
      line_content = line_content.replace(ending, '', 1)
      break

  # right align runtimes

  if line_content.endswith('s)'):
    div = line_content.rfind(' (')
    line_content = '%-53s%6s  ' % (line_content[:div], line_content[div + 2:-1])

  # skipped tests have extra single quotes around the reason
  if line_type == LineType.SKIPPED:
    line_content = line_content.replace("'(", "(", 1).replace(")'", ")", 1)

  if line_type == LineType.OK:
    new_ending = 'SUCCESS'
  elif line_type in (LineType.FAIL, LineType.ERROR):
    new_ending = 'FAILURE'
  elif line_type == LineType.SKIPPED:
    new_ending = 'SKIPPED'
  else:
    assert False, 'Unexpected line type: %s' % line_type
    return line_content

  if COLOR_SUPPORT:
    return '%-61s[%s]' % (line_content, term.format(new_ending, term.Attr.BOLD))
  else:
    return '%-61s[%s]' % (line_content, term.format(new_ending))


class ErrorTracker(object):
  """
  Stores any failure or error results we've encountered.
  """

  def __init__(self):
    self._errors = []
    self._error_modules = set()
    self._category = None
    self._error_noted = False

  def register_error(self):
    """
    If called then has_errors_occured() will report that an error has occured,
    even if we haven't encountered an error message in the tests.
    """

    self._error_noted = True

  def set_category(self, category):
    """
    Optional label that will be presented with testing failures until another
    category is specified. If set to None then no category labels are included.

    For tests with a lot of output this is intended to help narrow the haystack
    in which the user needs to look for failures. In practice this is mostly
    used to specify the integ target we're running under.

    :param str category: category to label errors as being under
    """

    self._category = category

  def has_errors_occured(self):
    return self._error_noted or bool(self._errors)

  def get_filter(self):
    def _error_tracker(line_type, line_content):
      if line_type in (LineType.FAIL, LineType.ERROR):
        if self._category:
          self._errors.append('[%s] %s' % (self._category, line_content))
        else:
          self._errors.append(line_content)

        module_match = re.match('.*\\((test\\.\\S+)\\.\\S+\\).*', line_content)

        if module_match:
          self._error_modules.add(module_match.group(1))

      return line_content

    return _error_tracker

  def get_modules(self):
    return self._error_modules

  def __iter__(self):
    for error_line in self._errors:
      yield error_line


def _flatten(seq):
  # Flattens nested collections into a single list. For instance...
  #
  # >>> _flatten([1, [2, 3], 4])
  # [1, 2, 3, 4]

  result = []

  for item in seq:
    if (isinstance(item, (tuple, list))):
      result.extend(_flatten(item))
    else:
      result.append(item)

  return result



from stem.control import Controller

with Controller.from_port(port = 9051) as controller:
  controller.authenticate()  # provide the password here if you set one

  bytes_read = controller.get_info("traffic/read")
  bytes_written = controller.get_info("traffic/written")

  print("My Tor relay has read %s bytes and written %s." % (bytes_read, bytes_written))



from stem.manual import Manual
from stem.util import term

try:
  print("Downloading tor's manual information, please wait...")
  manual = Manual.from_remote()
  print("  done\n")
except IOError as exc:
  print("  unsuccessful (%s), using information provided with stem\n" % exc)
  manual = Manual.from_cache()  # fall back to our bundled manual information

print('Which tor configuration would you like to learn about?  (press ctrl+c to quit)\n')

try:
  while True:
    requested_option = raw_input('> ').strip()

    if requested_option:
      if requested_option in manual.config_options:
        option = manual.config_options[requested_option]
        print(term.format('%s %s' % (option.name, option.usage), term.Color.GREEN, term.Attr.BOLD))
        print(term.format(option.summary, term.Color.GREEN))  # brief description provided by stem

        print(term.format('\nFull Description:\n', term.Color.GREEN, term.Attr.BOLD))
        print(term.format(option.description + '\n', term.Color.GREEN))
      else:
        print(term.format("Sorry, we don't have any information about %s. Are you sure it's an option?" % requested_option, term.Color.RED))
except KeyboardInterrupt:
  pass  # user pressed ctrl+c




import argparse
import collections
import time

import stem.connection
import stem.util.system
import stem.util.str_tools

from stem.control import Listener
from stem.util.connection import get_connections, port_usage, is_valid_ipv4_address

HEADER_LINE = " {version}   uptime: {uptime}   flags: {flags}\n"

DIV = '+%s+%s+%s+' % ('-' * 30, '-' * 6, '-' * 6)
COLUMN = '| %-28s | %4s | %4s |'

INBOUND_ORPORT = 'Inbound to our ORPort'
INBOUND_DIRPORT = 'Inbound to our DirPort'
INBOUND_CONTROLPORT = 'Inbound to our ControlPort'

OUTBOUND_ORPORT = 'Outbound to a relay'
OUTBOUND_EXIT = 'Outbound exit traffic'
OUTBOUND_UNKNOWN = 'Outbound uncategorized'


def main():
  parser = argparse.ArgumentParser()
  parser.add_argument("--ctrlport", help="default: 9051 or 9151")
  parser.add_argument("--resolver", help="default: autodetected")
  args = parser.parse_args()

  control_port = int(args.ctrlport) if args.ctrlport else 'default'
  controller = stem.connection.connect(control_port = ('127.0.0.1', control_port))

  if not controller:
    return

  desc = controller.get_network_status(default = None)
  pid = controller.get_pid()

  print(HEADER_LINE.format(
    version = str(controller.get_version()).split()[0],
    uptime = stem.util.str_tools.short_time_label(time.time() - stem.util.system.start_time(pid)),
    flags = ', '.join(desc.flags if desc else ['none']),
  ))

  policy = controller.get_exit_policy()
  relays = {}  # address => [orports...]

  for desc in controller.get_network_statuses():
    relays.setdefault(desc.address, []).append(desc.or_port)

  # categorize our connections

  categories = collections.OrderedDict((
    (INBOUND_ORPORT, []),
    (INBOUND_DIRPORT, []),
    (INBOUND_CONTROLPORT, []),
    (OUTBOUND_ORPORT, []),
    (OUTBOUND_EXIT, []),
    (OUTBOUND_UNKNOWN, []),
  ))

  exit_connections = {}  # port => [connections]

  for conn in get_connections(resolver = args.resolver, process_pid = pid):
    if conn.protocol == 'udp':
        continue

    if conn.local_port in controller.get_ports(Listener.OR, []):
      categories[INBOUND_ORPORT].append(conn)
    elif conn.local_port in controller.get_ports(Listener.DIR, []):
      categories[INBOUND_DIRPORT].append(conn)
    elif conn.local_port in controller.get_ports(Listener.CONTROL, []):
      categories[INBOUND_CONTROLPORT].append(conn)
    elif conn.remote_port in relays.get(conn.remote_address, []):
      categories[OUTBOUND_ORPORT].append(conn)
    elif policy.can_exit_to(conn.remote_address, conn.remote_port):
      categories[OUTBOUND_EXIT].append(conn)
      exit_connections.setdefault(conn.remote_port, []).append(conn)
    else:
      categories[OUTBOUND_UNKNOWN].append(conn)

  print(DIV)
  print(COLUMN % ('Type', 'IPv4', 'IPv6'))
  print(DIV)

  total_ipv4, total_ipv6 = 0, 0

  for label, connections in categories.items():
    if len(connections) == 0:
      continue

    ipv4_count = len([conn for conn in connections if is_valid_ipv4_address(conn.remote_address)])
    ipv6_count = len(connections) - ipv4_count

    total_ipv4, total_ipv6 = total_ipv4 + ipv4_count, total_ipv6 + ipv6_count
    print(COLUMN % (label, ipv4_count, ipv6_count))

  print(DIV)
  print(COLUMN % ('Total', total_ipv4, total_ipv6))
  print(DIV)
  print('')

  if exit_connections:
    print(DIV)
    print(COLUMN % ('Exit Port', 'IPv4', 'IPv6'))
    print(DIV)

    total_ipv4, total_ipv6 = 0, 0

    for port in sorted(exit_connections):
      connections = exit_connections[port]
      ipv4_count = len([conn for conn in connections if is_valid_ipv4_address(conn.remote_address)])
      ipv6_count = len(connections) - ipv4_count
      total_ipv4, total_ipv6 = total_ipv4 + ipv4_count, total_ipv6 + ipv6_count

      usage = port_usage(port)
      label = '%s (%s)' % (port, usage) if usage else port

      print(COLUMN % (label, ipv4_count, ipv6_count))

    print(DIV)
    print(COLUMN % ('Total', total_ipv4, total_ipv6))
    print(DIV)
    print('')


if __name__ == '__main__':
  main()



"""
Unit tests for stem.util.* contents.
"""

import datetime
import time
import unittest

from stem.util import datetime_to_unix

__all__ = [
  'conf',
  'connection',
  'enum',
  'log',
  'proc',
  'str_tools',
  'system',
  'tor_tools',
]


class TestBaseUtil(unittest.TestCase):
  def test_datetime_to_unix(self):
    self.assertEqual(1344251971.0, datetime_to_unix(datetime.datetime(2012, 8, 6, 11, 19, 31)))
    self.assertEqual(1515894416.0, datetime_to_unix(datetime.datetime(2018, 1, 14, 1, 46, 56)))
    self.assertTrue((time.time() - datetime_to_unix(datetime.datetime.utcnow())) < 2)



import stem.descriptor.remote

# Unlike the above example, this one downloads specifically through the
# ORPort of moria1 (long time tor directory authority).

try:
  consensus = stem.descriptor.remote.get_consensus(
    endpoints = (stem.ORPort('128.31.0.34', 9101),)
  )

  for desc in consensus:
    print("found relay %s (%s)" % (desc.nickname, desc.fingerprint))
except Exception as exc:
  print("Unable to retrieve the consensus: %s" % exc)



import socks  # SocksiPy module
import socket
import urllib

SOCKS_PORT = 7000

# Set socks proxy and wrap the urllib module

socks.setdefaultproxy(socks.PROXY_TYPE_SOCKS5, '127.0.0.1', SOCKS_PORT)
socket.socket = socks.socksocket

# Perform DNS resolution through the socket

def getaddrinfo(*args):
  return [(socket.AF_INET, socket.SOCK_STREAM, 6, '', (args[0], args[1]))]

socket.getaddrinfo = getaddrinfo

def query(url):
  """
  Uses urllib to fetch a site using SocksiPy for Tor over the SOCKS_PORT.
  """

  try:
    return urllib.urlopen(url).read()
  except:
    return "Unable to reach %s" % url



from stem.descriptor.remote import DescriptorDownloader
from stem.version import Version

downloader = DescriptorDownloader()
count, with_contact = 0, 0

print("Checking for outdated relays...")
print("")

for desc in downloader.get_server_descriptors():
  if desc.tor_version < Version('0.2.3.0'):
    count += 1

    if desc.contact:
      print('  %-15s %s' % (desc.tor_version, desc.contact.decode("utf-8", "replace")))
      with_contact += 1

print("")
print("%i outdated relays found, %i had contact information" % (count, with_contact))



"""
Integration tests for stem.descriptor.networkstatus.
"""

import os
import unittest

import stem
import stem.descriptor
import stem.descriptor.remote
import stem.util.test_tools
import stem.version
import test
import test.require
import test.runner

from stem.util.test_tools import asynchronous


class TestNetworkStatus(unittest.TestCase):
  @staticmethod
  def run_tests(args):
    stem.util.test_tools.ASYNC_TESTS['test.integ.descriptor.networkstatus.test_cached_consensus'].run(args.test_dir, threaded = True)
    stem.util.test_tools.ASYNC_TESTS['test.integ.descriptor.networkstatus.test_cached_microdesc_consensus'].run(args.test_dir, threaded = True)

  @test.require.only_run_once
  @test.require.online
  @test.require.cryptography
  def test_signature_validation(self):
    """
    The full consensus is pretty sizable so rather than storing a copy of it
    using the remote module. Chekcing the signature on the current consensus.
    """

    stem.descriptor.remote.get_consensus(document_handler = stem.descriptor.DocumentHandler.DOCUMENT, validate = True).run()

  @asynchronous
  def test_cached_consensus(test_dir):
    """
    Parses the cached-consensus file in our data directory.
    """

    consensus_path = os.path.join(test_dir, 'cached-consensus')

    if not os.path.exists(consensus_path):
      raise stem.util.test_tools.SkipTest('(no cached-consensus)')
    elif stem.util.system.is_windows():
      # Unable to check memory usage on windows, so can't prevent hanging the
      # system if things go bad.

      raise stem.util.test_tools.SkipTest('(unavailable on windows)')

    count, reported_flags = 0, []

    with open(consensus_path, 'rb') as descriptor_file:
      for router in stem.descriptor.parse_file(descriptor_file, 'network-status-consensus-3 1.0', validate = True):
        count += 1

        for flag in router.flags:
          if flag not in stem.Flag and flag not in reported_flags:
            test.register_new_capability('Flag (consensus)', flag)
            reported_flags.append(flag)

        for line in router.get_unrecognized_lines():
          test.register_new_capability('Consensus Line', line, suppression_token = line.split()[0])

    # Sanity test that there's at least a hundred relays. If that's not the
    # case then this probably isn't a real, complete tor consensus.

    if count < 100:
      raise AssertionError('%s only included %s relays' % (consensus_path, count))

  @asynchronous
  def test_cached_microdesc_consensus(test_dir):
    """
    Parses the cached-microdesc-consensus file in our data directory.
    """

    consensus_path = os.path.join(test_dir, 'cached-microdesc-consensus')

    if not os.path.exists(consensus_path):
      raise stem.util.test_tools.SkipTest('(no cached-microdesc-consensus)')
    elif stem.util.system.is_windows():
      raise stem.util.test_tools.SkipTest('(unavailable on windows)')

    count, reported_flags = 0, []

    with open(consensus_path, 'rb') as descriptor_file:
      for router in stem.descriptor.parse_file(descriptor_file, 'network-status-microdesc-consensus-3 1.0', validate = True):
        count += 1

        for flag in router.flags:
          if flag not in stem.Flag:
            test.register_new_capability('Flag (microdescriptor consensus)', flag)
            reported_flags.append(flag)

        for line in router.get_unrecognized_lines():
          test.register_new_capability('Microdescriptor Consensus Line', line, suppression_token = line.split()[0])

    if count < 100:
      raise AssertionError('%s only included %s relays' % (consensus_path, count))



"""
Unit tests for the stem.connection.connect function.
"""

import unittest

try:
  from StringIO import StringIO
except ImportError:
  from io import StringIO

try:
  from mock import Mock, patch
except ImportError:
  from unittest.mock import Mock, patch

import stem
import stem.connection
import stem.socket


class TestConnect(unittest.TestCase):
  @patch('sys.stdout', new_callable = StringIO)
  @patch('stem.util.system.is_running')
  @patch('os.path.exists', Mock(return_value = True))
  @patch('stem.socket.ControlSocketFile', Mock(side_effect = stem.SocketError('failed')))
  @patch('stem.socket.ControlPort', Mock(side_effect = stem.SocketError('failed')))
  @patch('stem.connection._connect_auth', Mock())
  def test_failue_with_the_default_endpoint(self, is_running_mock, stdout_mock):
    is_running_mock.return_value = False
    self._assert_connect_fails_with({}, stdout_mock, "Unable to connect to tor. Are you sure it's running?")

    is_running_mock.return_value = True
    self._assert_connect_fails_with({}, stdout_mock, "Unable to connect to tor. Maybe it's running without a ControlPort?")

  @patch('sys.stdout', new_callable = StringIO)
  @patch('os.path.exists')
  @patch('stem.util.system.is_running', Mock(return_value = True))
  @patch('stem.socket.ControlSocketFile', Mock(side_effect = stem.SocketError('failed')))
  @patch('stem.socket.ControlPort', Mock(side_effect = stem.SocketError('failed')))
  @patch('stem.connection._connect_auth', Mock())
  def test_failure_with_a_custom_endpoint(self, path_exists_mock, stdout_mock):
    path_exists_mock.return_value = True
    self._assert_connect_fails_with({'control_port': ('127.0.0.1', 80), 'control_socket': None}, stdout_mock, "Unable to connect to 127.0.0.1:80: failed")
    self._assert_connect_fails_with({'control_port': None, 'control_socket': '/tmp/my_socket'}, stdout_mock, "Unable to connect to '/tmp/my_socket': failed")

    path_exists_mock.return_value = False
    self._assert_connect_fails_with({'control_port': ('127.0.0.1', 80), 'control_socket': None}, stdout_mock, "Unable to connect to 127.0.0.1:80: failed")
    self._assert_connect_fails_with({'control_port': None, 'control_socket': '/tmp/my_socket'}, stdout_mock, "The socket file you specified (/tmp/my_socket) doesn't exist")

  @patch('stem.socket.ControlPort')
  @patch('os.path.exists', Mock(return_value = False))
  @patch('stem.connection._connect_auth', Mock())
  def test_getting_a_control_port(self, port_mock):
    stem.connection.connect()
    port_mock.assert_called_once_with('127.0.0.1', 9051)
    port_mock.reset_mock()

    stem.connection.connect(control_port = ('255.0.0.10', 80), control_socket = None)
    port_mock.assert_called_once_with('255.0.0.10', 80)

  @patch('stem.socket.ControlSocketFile')
  @patch('os.path.exists', Mock(return_value = True))
  @patch('stem.connection._connect_auth', Mock())
  def test_getting_a_control_socket(self, socket_mock):
    stem.connection.connect()
    socket_mock.assert_called_once_with('/var/run/tor/control')
    socket_mock.reset_mock()

    stem.connection.connect(control_port = None, control_socket = '/tmp/my_socket')
    socket_mock.assert_called_once_with('/tmp/my_socket')

  def _assert_connect_fails_with(self, args, stdout_mock, msg):
    result = stem.connection.connect(**args)

    if result is not None:
      self.fail()

    # Python 3.x seems to have an oddity where StringIO has prefixed null
    # characters (\x00) after we call truncate(). This could be addressed
    # a couple ways...
    #
    #   * Don't use a stdout mock more than once.
    #   * Strip the null characters.
    #
    # Opting for the second (which is admittedly a hack) so the tests are a
    # little nicer.

    stdout_output = stdout_mock.getvalue()
    stdout_mock.truncate(0)
    self.assertEqual(msg, stdout_output.strip().lstrip('\x00'))

  @patch('stem.connection.authenticate')
  def test_auth_success(self, authenticate_mock):
    control_socket = Mock()

    stem.connection._connect_auth(control_socket, None, False, None, None)
    authenticate_mock.assert_called_with(control_socket, None, None)
    authenticate_mock.reset_mock()

    stem.connection._connect_auth(control_socket, 's3krit!!!', False, '/my/chroot', None)
    authenticate_mock.assert_called_with(control_socket, 's3krit!!!', '/my/chroot')

  @patch('getpass.getpass')
  @patch('stem.connection.authenticate')
  def test_auth_success_with_password_prompt(self, authenticate_mock, getpass_mock):
    control_socket = Mock()

    def authenticate_mock_func(controller, password, *args):
      if password is None:
        raise stem.connection.MissingPassword('no password')
      elif password == 'my_password':
        return None  # success
      else:
        raise ValueError('Unexpected authenticate_mock input: %s' % password)

    authenticate_mock.side_effect = authenticate_mock_func
    getpass_mock.return_value = 'my_password'

    stem.connection._connect_auth(control_socket, None, True, None, None)
    authenticate_mock.assert_any_call(control_socket, None, None)
    authenticate_mock.assert_any_call(control_socket, 'my_password', None)

  @patch('sys.stdout', new_callable = StringIO)
  @patch('stem.connection.authenticate')
  def test_auth_failure(self, authenticate_mock, stdout_mock):
    control_socket = stem.socket.ControlPort(connect = False)

    authenticate_mock.side_effect = stem.connection.IncorrectSocketType('unable to connect to socket')
    self._assert_authenticate_fails_with(control_socket, stdout_mock, 'Please check in your torrc that 9051 is the ControlPort.')

    control_socket = stem.socket.ControlSocketFile(connect = False)

    self._assert_authenticate_fails_with(control_socket, stdout_mock, 'Are you sure the interface you specified belongs to')

    authenticate_mock.side_effect = stem.connection.UnrecognizedAuthMethods('unable to connect', ['telepathy'])
    self._assert_authenticate_fails_with(control_socket, stdout_mock, 'Tor is using a type of authentication we do not recognize...\n\n  telepathy')

    authenticate_mock.side_effect = stem.connection.IncorrectPassword('password rejected')
    self._assert_authenticate_fails_with(control_socket, stdout_mock, 'Incorrect password')

    authenticate_mock.side_effect = stem.connection.UnreadableCookieFile('permission denied', '/tmp/my_cookie', False)
    self._assert_authenticate_fails_with(control_socket, stdout_mock, "We were unable to read tor's authentication cookie...\n\n  Path: /tmp/my_cookie\n  Issue: permission denied")

    authenticate_mock.side_effect = stem.connection.OpenAuthRejected('crazy failure')
    self._assert_authenticate_fails_with(control_socket, stdout_mock, 'Unable to authenticate: crazy failure')

  def _assert_authenticate_fails_with(self, control_socket, stdout_mock, msg):
    result = stem.connection._connect_auth(control_socket, None, False, None, None)

    if result is not None:
      self.fail()  # _connect_auth() was successful

    stdout_output = stdout_mock.getvalue()
    stdout_mock.truncate(0)

    if msg not in stdout_output:
      self.fail("Expected...\n\n%s\n\n... which couldn't be found in...\n\n%s" % (msg, stdout_output))



import time
import stem.descriptor

def measure_average_advertised_bandwidth(path):
  start_time = time.time()
  total_bw, count = 0, 0

  for desc in stem.descriptor.parse_file(path):
    total_bw += min(desc.average_bandwidth, desc.burst_bandwidth, desc.observed_bandwidth)
    count += 1

  runtime = time.time() - start_time
  print("Finished measure_average_advertised_bandwidth('%s')" % path)
  print('  Total time: %i seconds' % runtime)
  print('  Processed server descriptors: %i' % count)
  print('  Average advertised bandwidth: %i' % (total_bw / count))
  print('  Time per server descriptor: %0.5f seconds' % (runtime / count))
  print('')

def measure_countries_v3_requests(path):
  start_time = time.time()
  countries, count = set(), 0

  for desc in stem.descriptor.parse_file(path):
    if desc.dir_v3_responses:
      countries.update(desc.dir_v3_responses.keys())

    count += 1

  runtime = time.time() - start_time
  print("Finished measure_countries_v3_requests('%s')" % path)
  print('  Total time: %i seconds' % runtime)
  print('  Processed extrainfo descriptors: %i' % count)
  print('  Number of countries: %i' % len(countries))
  print('  Time per extrainfo descriptor: %0.5f seconds' % (runtime / count))
  print('')

def measure_average_relays_exit(path):
  start_time = time.time()
  total_relays, exits, consensuses = 0, 0, 0

  for consensus in stem.descriptor.parse_file(path, document_handler = stem.descriptor.DocumentHandler.DOCUMENT):
    for desc in consensus.routers.values():
      if 'Exit' in desc.flags:
        exits += 1

      total_relays += 1

    consensuses += 1

  runtime = time.time() - start_time
  print("Finished measure_average_relays_exit('%s')" % path)
  print('  Total time: %i seconds' % runtime)
  print('  Processed %i consensuses with %i router status entries' % (consensuses, total_relays))
  print('  Total exits: %i (%0.2f%%)' % (exits, float(exits) / total_relays))
  print('  Time per consensus: %0.5f seconds' % (runtime / consensuses))
  print('')

def measure_fraction_relays_exit_80_microdescriptors(path):
  start_time = time.time()
  exits, count = 0, 0

  for desc in stem.descriptor.parse_file(path):
    if desc.exit_policy.can_exit_to(port = 80):
      exits += 1

    count += 1

  runtime = time.time() - start_time
  print("Finished measure_fraction_relays_exit_80_microdescriptors('%s')" % path)
  print('  Total time: %i seconds' % runtime)
  print('  Processed microdescriptors: %i' % count)
  print('  Total exits to port 80: %i (%0.2f%%)' % (exits, float(exits) / count))
  print('  Time per microdescriptor: %0.5f seconds' % (runtime / count))
  print('')

measure_average_advertised_bandwidth('/home/atagar/Desktop/server-descriptors-2015-11.tar')
measure_countries_v3_requests('/home/atagar/Desktop/extra-infos-2015-11.tar')
measure_average_relays_exit('/home/atagar/Desktop/consensuses-2015-11.tar')
measure_fraction_relays_exit_80_microdescriptors('/home/atagar/Desktop/microdescs-2015-11.tar')




"""
Integration tests for establishing a connection with tor's ORPort.
"""

import time
import unittest

import stem
import test.runner

from stem.client import Relay


class TestConnection(unittest.TestCase):
  def test_invalid_arguments(self):
    """
    Provide invalid arguments to Relay.connect().
    """

    self.assertRaisesWith(ValueError, "'nope' isn't an IPv4 or IPv6 address", Relay.connect, 'nope', 80)
    self.assertRaisesWith(ValueError, "'-54' isn't a valid port", Relay.connect, '127.0.0.1', -54)
    self.assertRaisesWith(ValueError, "Connection can't be established without a link protocol.", Relay.connect, '127.0.0.1', 54, [])

  def test_not_orport(self):
    """
    Attempt to connect to an ORPort that doesn't exist.
    """

    self.assertRaisesWith(stem.SocketError, "Failed to connect to 127.0.0.1:1587. Maybe it isn't an ORPort?", Relay.connect, '127.0.0.1', 1587)

    # connect to our ControlPort like it's an ORPort

    if test.runner.Torrc.PORT in test.runner.get_runner().get_options():
      self.assertRaisesWith(stem.SocketError, "Failed to SSL authenticate to 127.0.0.1:1111. Maybe it isn't an ORPort?", Relay.connect, '127.0.0.1', test.runner.CONTROL_PORT)

  def test_no_common_link_protocol(self):
    """
    Connection without a commonly accepted link protocol version.
    """

    for link_protocol in (1, 2, 6, 20):
      self.assertRaisesWith(stem.SocketError, 'Unable to establish a common link protocol with 127.0.0.1:1113', Relay.connect, '127.0.0.1', test.runner.ORPORT, [link_protocol])

  def test_connection_time(self):
    """
    Checks duration we've been connected.
    """

    before = time.time()

    with Relay.connect('127.0.0.1', test.runner.ORPORT) as conn:
      connection_time = conn.connection_time()
      self.assertTrue(time.time() >= connection_time >= before)
      time.sleep(0.02)
      self.assertTrue(conn.is_alive())

    self.assertFalse(conn.is_alive())
    self.assertTrue(conn.connection_time() >= connection_time + 0.02)

  def test_established(self):
    """
    Successfully establish ORPort connection.
    """

    conn = Relay.connect('127.0.0.1', test.runner.ORPORT)
    self.assertTrue(int(conn.link_protocol) in (4, 5))



"""
Integration tests for the stem.util.system functions against the tor process
that we're running.
"""

import getpass
import os
import tempfile
import unittest

import stem.prereq
import stem.util.proc
import stem.util.system
import test.require
import test.runner

from stem.util.system import State, DaemonTask

try:
  # added in python 3.3
  from unittest.mock import Mock, patch
except ImportError:
  from mock import Mock, patch


def filter_system_call(prefixes):
  """
  Provides a functor that passes calls on to the stem.util.system.call()
  function if it matches one of the prefixes, and acts as a no-op otherwise.
  """

  original_call = stem.util.system.call

  def _filter_system_call(command, default):
    for prefix in prefixes:
      if command.startswith(prefix):
        real_call_function = original_call
        return real_call_function(command)

  return _filter_system_call


def _is_single_tor_running():
  if stem.util.system.is_windows():
    return True  # TODO: not sure how to check for this on windows
  elif not stem.util.system.is_bsd():
    tor_cmd = test.runner.get_runner().get_tor_command(True)
    pgrep_results = stem.util.system.call(stem.util.system.GET_PID_BY_NAME_PGREP % tor_cmd)
    return len(pgrep_results) == 1
  else:
    ps_results = stem.util.system.call(stem.util.system.GET_PID_BY_NAME_PS_BSD)
    results = [r for r in ps_results if r.endswith(' tor')]
    return len(results) == 1


def _is_linux():
  return not stem.util.system.is_bsd() and not stem.util.system.is_windows()


def _has_port():
  """
  True if our test runner has a control port, False otherwise.
  """

  return test.runner.Torrc.PORT in test.runner.get_runner().get_options()


require_control_port = test.require.needs(_has_port, 'test instance has no port')
require_linux = test.require.needs(_is_linux, 'linux only')
require_bsd = test.require.needs(stem.util.system.is_bsd, 'bsd only')
require_path = test.require.needs(lambda: 'PATH' in os.environ, 'requires PATH')


def require_single_tor_instance(func):
  # Checking both before and after the test to see if we're running only a
  # single tor instance. We do both to narrow the possability of the test
  # failing due to a race.

  def wrapped(self, *args, **kwargs):
    if _is_single_tor_running():
      try:
        return func(self, *args, **kwargs)
      except:
        if _is_single_tor_running():
          raise
        else:
          self.skipTest('(multiple tor instances)')
    else:
      self.skipTest('(multiple tor instances)')

  return wrapped


class TestSystem(unittest.TestCase):
  def test_daemon_task_when_successful(self):
    """
    Checks a simple, successfully DaemonTask that simply echos a value.
    """

    task = DaemonTask(lambda arg: arg, ('hello world',))

    self.assertEqual(None, task.result)
    self.assertEqual(State.PENDING, task.status)

    task.run()
    self.assertEqual('hello world', task.join())
    self.assertEqual(State.DONE, task.status)
    self.assertTrue(0 < task.runtime < 1.0)

  def test_daemon_task_on_failure(self):
    """
    Checks DaemonTask when an exception is raised.
    """

    def _test_task(arg):
      raise RuntimeError(arg)

    task = DaemonTask(_test_task, ('hello world',))

    self.assertEqual(None, task.result)
    self.assertEqual(State.PENDING, task.status)

    task.run()
    self.assertRaisesWith(RuntimeError, 'hello world', task.join)
    self.assertEqual(State.FAILED, task.status)
    self.assertTrue(0 < task.runtime < 1.0)

  @require_path
  def test_is_available(self):
    """
    Checks the stem.util.system.is_available function.
    """

    # I have yet to see a platform without 'ls'

    if stem.util.system.is_windows():
      self.assertTrue(stem.util.system.is_available('dir'))
    else:
      self.assertTrue(stem.util.system.is_available('ls'))

    # but it would be kinda weird if this did...

    self.assertFalse(stem.util.system.is_available('blarg_and_stuff'))

  def test_is_running_by_pid(self):
    """
    Checks the stem.util.system.is_running function with a pid.
    """

    self.assertTrue(stem.util.system.is_running(test.runner.get_runner().get_pid()))
    self.assertFalse(stem.util.system.is_running(528955))

  @test.require.command('ps')
  def test_is_running_by_name(self):
    """
    Checks the stem.util.system.is_running function with a process name.
    """

    # Check to see if the command we started tor with is running. The process
    # might be running under another name so need to check for 'tor.real' too
    # (#15449).

    tor_cmd = test.runner.get_runner().get_tor_command(True)
    self.assertTrue(stem.util.system.is_running(tor_cmd) or stem.util.system.is_running('tor.real'))
    self.assertFalse(stem.util.system.is_running('blarg_and_stuff'))

  @require_path
  @require_single_tor_instance
  def test_pid_by_name(self):
    """
    Checks general usage of the stem.util.system.pid_by_name function. This
    will fail if there's other tor instances running.
    """

    tor_pid = test.runner.get_runner().get_pid()
    tor_cmd = test.runner.get_runner().get_tor_command(True)
    self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))
    self.assertEqual(None, stem.util.system.pid_by_name('blarg_and_stuff'))

  @require_single_tor_instance
  @test.require.command('pgrep')
  def test_pid_by_name_pgrep(self):
    """
    Tests the pid_by_name function with a pgrep response.
    """

    pgrep_prefix = stem.util.system.GET_PID_BY_NAME_PGREP % ''
    call_replacement = filter_system_call([pgrep_prefix])

    with patch('stem.util.system.call') as call_mock:
      call_mock.side_effect = call_replacement

      tor_pid = test.runner.get_runner().get_pid()
      tor_cmd = test.runner.get_runner().get_tor_command(True)
      self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))

  @require_single_tor_instance
  @test.require.command('pidof')
  def test_pid_by_name_pidof(self):
    """
    Tests the pid_by_name function with a pidof response.
    """

    pidof_prefix = stem.util.system.GET_PID_BY_NAME_PIDOF % ''
    call_replacement = filter_system_call([pidof_prefix])

    with patch('stem.util.system.call') as call_mock:
      call_mock.side_effect = call_replacement

      tor_pid = test.runner.get_runner().get_pid()
      tor_cmd = test.runner.get_runner().get_tor_command()
      self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))

  @require_linux
  @require_single_tor_instance
  @test.require.command('ps')
  def test_pid_by_name_ps_linux(self):
    """
    Tests the pid_by_name function with the linux variant of ps.
    """

    ps_prefix = stem.util.system.GET_PID_BY_NAME_PS_LINUX % ''
    call_replacement = filter_system_call([ps_prefix])

    with patch('stem.util.system.call') as call_mock:
      call_mock.side_effect = call_replacement

      tor_pid = test.runner.get_runner().get_pid()
      tor_cmd = test.runner.get_runner().get_tor_command(True)
      self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))

  @require_bsd
  @require_single_tor_instance
  @test.require.command('ps')
  def test_pid_by_name_ps_bsd(self):
    """
    Tests the pid_by_name function with the bsd variant of ps.
    """

    ps_prefix = stem.util.system.GET_PID_BY_NAME_PS_BSD
    call_replacement = filter_system_call([ps_prefix])

    with patch('stem.util.system.call') as call_mock:
      call_mock.side_effect = call_replacement

      tor_pid = test.runner.get_runner().get_pid()
      tor_cmd = test.runner.get_runner().get_tor_command(True)
      self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))

  @require_single_tor_instance
  @test.require.ptrace
  @test.require.command('lsof')
  def test_pid_by_name_lsof(self):
    """
    Tests the pid_by_name function with a lsof response.
    """

    lsof_prefix = stem.util.system.GET_PID_BY_NAME_LSOF % ''
    call_replacement = filter_system_call([lsof_prefix])

    with patch('stem.util.system.call') as call_mock:
      call_mock.side_effect = call_replacement

      tor_cmd = test.runner.get_runner().get_tor_command(True)
      our_tor_pid = test.runner.get_runner().get_pid()
      all_tor_pids = stem.util.system.pid_by_name(tor_cmd, multiple = True)

      if len(all_tor_pids) == 1:
        self.assertEqual(our_tor_pid, all_tor_pids[0])

  @require_single_tor_instance
  @test.require.command('tasklist')
  def test_pid_by_name_tasklist(self):
    """
    Tests the pid_by_name function with a tasklist response.
    """

    runner = test.runner.get_runner()
    self.assertEqual(runner.get_pid(), stem.util.system.pid_by_name(runner.get_tor_command(True)))

  @require_control_port
  @test.require.ptrace
  def test_pid_by_port(self):
    """
    Checks general usage of the stem.util.system.pid_by_port function.
    """

    if stem.util.system.is_windows():
      self.skipTest('(unavailable on windows)')
      return
    elif stem.util.system.is_mac() or stem.util.system.is_gentoo():
      self.skipTest('(resolvers unavailable)')
      return
    elif not stem.util.system.is_available('netstat') or \
             stem.util.system.is_available('sockstat') or \
              stem.util.system.is_available('lsof'):
      self.skipTest('(connection resolvers unavailable)')
      return

    runner = test.runner.get_runner()
    tor_pid, tor_port = runner.get_pid(), test.runner.CONTROL_PORT
    self.assertEqual(tor_pid, stem.util.system.pid_by_port(tor_port))
    self.assertEqual(None, stem.util.system.pid_by_port(99999))

  @require_linux
  @require_control_port
  @test.require.ptrace
  @test.require.command('netstat')
  def test_pid_by_port_netstat(self):
    """
    Tests the pid_by_port function with a netstat response.
    """

    if stem.util.system.is_gentoo():
      self.skipTest('(unavailable on gentoo)')
      return

    netstat_prefix = stem.util.system.GET_PID_BY_PORT_NETSTAT

    call_replacement = filter_system_call([netstat_prefix])

    with patch('stem.util.system.call') as call_mock:
      call_mock.side_effect = call_replacement

      tor_pid = test.runner.get_runner().get_pid()
      self.assertEqual(tor_pid, stem.util.system.pid_by_port(test.runner.CONTROL_PORT))

  @require_bsd
  @require_control_port
  @test.require.ptrace
  @test.require.command('sockstat')
  def test_pid_by_port_sockstat(self):
    """
    Tests the pid_by_port function with a sockstat response.
    """

    sockstat_prefix = stem.util.system.GET_PID_BY_PORT_SOCKSTAT % ''
    call_replacement = filter_system_call([sockstat_prefix])

    with patch('stem.util.system.call') as call_mock:
      call_mock.side_effect = call_replacement

      tor_pid = test.runner.get_runner().get_pid()
      self.assertEqual(tor_pid, stem.util.system.pid_by_port(test.runner.CONTROL_PORT))

  @require_control_port
  @test.require.ptrace
  @test.require.command('lsof')
  def test_pid_by_port_lsof(self):
    """
    Tests the pid_by_port function with a lsof response.
    """

    if stem.util.system.is_mac() or stem.util.system.is_gentoo():
      self.skipTest('(resolvers unavailable)')
      return

    lsof_prefix = stem.util.system.GET_PID_BY_PORT_LSOF

    call_replacement = filter_system_call([lsof_prefix])

    with patch('stem.util.system.call') as call_mock:
      call_mock.side_effect = call_replacement

      tor_pid = test.runner.get_runner().get_pid()
      self.assertEqual(tor_pid, stem.util.system.pid_by_port(test.runner.CONTROL_PORT))

  def test_pid_by_open_file(self):
    """
    Checks the stem.util.system.pid_by_open_file function.
    """

    # check a directory that exists, but isn't claimed by any application
    tmpdir = tempfile.mkdtemp()
    self.assertEqual(None, stem.util.system.pid_by_open_file(tmpdir))

    # check a directory that doesn't exist
    os.rmdir(tmpdir)
    self.assertEqual(None, stem.util.system.pid_by_open_file(tmpdir))

  @require_path
  def test_pids_by_user(self):
    """
    Checks the stem.util.system.pids_by_user function.
    """

    # our own pid should be among the processes for our user

    pids = stem.util.system.pids_by_user(getpass.getuser())
    self.assertTrue(os.getpid() in pids)

  @test.require.ptrace
  def test_cwd(self):
    """
    Checks general usage of the stem.util.system.cwd function.
    """

    if stem.util.system.is_windows():
      self.skipTest('(unavailable on windows)')
      return

    runner = test.runner.get_runner()
    runner_pid, tor_cwd = runner.get_pid(), runner.get_tor_cwd()
    self.assertEqual(tor_cwd, stem.util.system.cwd(runner_pid))
    self.assertEqual(None, stem.util.system.cwd(99999))

  @test.require.ptrace
  @test.require.command('pwdx')
  def test_cwd_pwdx(self):
    """
    Tests the pid_by_cwd function with a pwdx response.
    """

    # filter the call function to only allow this command

    pwdx_prefix = stem.util.system.GET_CWD_PWDX % ''

    call_replacement = filter_system_call([pwdx_prefix])

    with patch('stem.util.system.call') as call_mock:
      call_mock.side_effect = call_replacement

      runner = test.runner.get_runner()
      runner_pid, tor_cwd = runner.get_pid(), runner.get_tor_cwd()
      self.assertEqual(tor_cwd, stem.util.system.cwd(runner_pid))

  @test.require.ptrace
  @test.require.command('lsof')
  def test_cwd_lsof(self):
    """
    Tests the pid_by_cwd function with a lsof response.
    """

    # filter the call function to only allow this command

    lsof_prefix = 'lsof -a -p '

    call_replacement = filter_system_call([lsof_prefix])

    with patch('stem.util.system.call') as call_mock:
      call_mock.side_effect = call_replacement

      runner = test.runner.get_runner()
      runner_pid, tor_cwd = runner.get_pid(), runner.get_tor_cwd()
      self.assertEqual(tor_cwd, stem.util.system.cwd(runner_pid))

  def test_user_none(self):
    """
    Tests the user function when the process doesn't exist.
    """

    self.assertEqual(None, stem.util.system.user(None))
    self.assertEqual(None, stem.util.system.user(-5))
    self.assertEqual(None, stem.util.system.start_time(98765))

  @test.require.proc
  def test_user_proc(self):
    """
    Tests the user function with a proc response.
    """

    call_replacement = filter_system_call(['ps '])

    with patch('stem.util.system.call') as call_mock:
      call_mock.side_effect = call_replacement

      # we started our tor process so it should be running with the same user

      pid = test.runner.get_runner().get_pid()
      self.assertTrue(getpass.getuser(), stem.util.system.user(pid))

  @test.require.command('ps')
  @patch('stem.util.proc.is_available', Mock(return_value = False))
  def test_user_ps(self):
    """
    Tests the user function with a ps response.
    """

    pid = test.runner.get_runner().get_pid()
    self.assertTrue(getpass.getuser(), stem.util.system.user(pid))

  def test_start_time_none(self):
    """
    Tests the start_time function when the process doesn't exist.
    """

    self.assertEqual(None, stem.util.system.start_time(None))
    self.assertEqual(None, stem.util.system.start_time(-5))
    self.assertEqual(None, stem.util.system.start_time(98765))

  @test.require.proc
  def test_start_time_proc(self):
    """
    Tests the start_time function with a proc response.
    """

    call_replacement = filter_system_call(['ps '])

    with patch('stem.util.system.call') as call_mock:
      call_mock.side_effect = call_replacement

      pid = test.runner.get_runner().get_pid()
      self.assertTrue(stem.util.system.start_time(pid) >= 0)

  @test.require.command('ps')
  @patch('stem.util.proc.is_available', Mock(return_value = False))
  def test_start_time_ps(self):
    """
    Tests the start_time function with a ps response.
    """

    pid = test.runner.get_runner().get_pid()
    self.assertTrue(stem.util.system.start_time(pid) >= 0)

  def test_bsd_jail_id(self):
    """
    Exercises the stem.util.system.bsd_jail_id function, running through
    the failure case (since I'm not on BSD I can't really test this function
    properly).
    """

    self.assertEqual(0, stem.util.system.bsd_jail_id(99999))

  def test_expand_path(self):
    """
    Exercises the expand_path() method with actual runtime data.
    """

    # Some of the following tests get confused when ran as root. For instance,
    # in my case...
    #
    #   >>> os.path.expanduser('~')
    #   '/home/atagar'
    #
    #   >>> os.path.expanduser('~root')
    #   '/root'

    if getpass.getuser() == 'root':
      self.skipTest('(running as root)')
      return

    self.assertEqual(os.getcwd(), stem.util.system.expand_path('.'))
    self.assertEqual(os.getcwd(), stem.util.system.expand_path('./'))
    self.assertEqual(os.path.join(os.getcwd(), 'foo'), stem.util.system.expand_path('./foo'))

    home_dir, username = os.path.expanduser('~'), getpass.getuser()
    self.assertEqual(home_dir, stem.util.system.expand_path('~'))
    self.assertEqual(home_dir, stem.util.system.expand_path('~/'))
    self.assertEqual(home_dir, stem.util.system.expand_path('~%s' % username))
    self.assertEqual(os.path.join(home_dir, 'foo'), stem.util.system.expand_path('~%s/foo' % username))

  def test_call_timeout(self):
    self.assertRaisesWith(stem.util.system.CallTimeoutError, "Process didn't finish after 0.0 seconds", stem.util.system.call, 'sleep 1', timeout = 0.001)

  def test_call_time_tracked(self):
    """
    Check that time taken in the call() function is tracked by SYSTEM_CALL_TIME.
    """

    initial = stem.util.system.SYSTEM_CALL_TIME
    stem.util.system.call('sleep 0.005')
    self.assertTrue(stem.util.system.SYSTEM_CALL_TIME - initial > 0.004)

  def test_set_process_name(self):
    """
    Exercises the get_process_name() and set_process_name() methods.
    """

    if stem.prereq.is_pypy():
      self.skipTest('(unimplemented for pypy)')
      return

    initial_name = stem.util.system.get_process_name()
    self.assertTrue('run_tests.py' in initial_name)

    try:
      stem.util.system.set_process_name('stem_integ')
      self.assertEqual('stem_integ', stem.util.system.get_process_name())
    finally:
      stem.util.system.set_process_name(initial_name)



"""
Integration tests for the stem.socket.ControlSocket subclasses. When ran these
test basic functionality for a ControlPort *or* ControlSocketFile, depending on
which can connect to our tor instance.

These tests share a similar domain with those for the ControlMessage, but where
those focus on parsing and correctness of the content these are more concerned
with the behavior of the socket itself.
"""

import time
import unittest

import stem.connection
import stem.control
import stem.socket
import test
import test.require
import test.runner


class TestControlSocket(unittest.TestCase):
  @test.require.controller
  def test_connection_time(self):
    """
    Checks that our connection_time method tracks when our state's changed.
    """

    test_start = time.time()
    runner = test.runner.get_runner()

    with runner.get_tor_socket() as control_socket:
      connection_time = control_socket.connection_time()

      # connection time should be between our tests start and now

      self.assertTrue(test_start <= connection_time <= time.time())

      # connection time should be absolute (shouldn't change as time goes on)

      time.sleep(0.001)
      self.assertEqual(connection_time, control_socket.connection_time())

      # should change to the disconnection time if we detactch

      control_socket.close()
      disconnection_time = control_socket.connection_time()
      self.assertTrue(connection_time < disconnection_time <= time.time())

      # then change again if we reconnect

      time.sleep(0.001)
      control_socket.connect()
      reconnection_time = control_socket.connection_time()
      self.assertTrue(disconnection_time < reconnection_time <= time.time())

  @test.require.controller
  def test_send_buffered(self):
    """
    Sends multiple requests before receiving back any of the replies.
    """

    runner = test.runner.get_runner()

    with runner.get_tor_socket() as control_socket:
      for _ in range(100):
        control_socket.send('GETINFO version')

      for _ in range(100):
        response = control_socket.recv()
        self.assertTrue(str(response).startswith('version=%s' % test.tor_version()))
        self.assertTrue(str(response).endswith('\nOK'))

  @test.require.controller
  def test_send_closed(self):
    """
    Sends a message after we've closed the connection.
    """

    with test.runner.get_runner().get_tor_socket() as control_socket:
      self.assertTrue(control_socket.is_alive())
      control_socket.close()
      self.assertFalse(control_socket.is_alive())

      self.assertRaises(stem.SocketClosed, control_socket.send, 'blarg')

  @test.require.controller
  def test_send_disconnected(self):
    """
    Sends a message to a socket that has been disconnected by the other end.

    Our behavior upon disconnection slightly differs based on if we're a port
    or socket file based connection. With a control port we won't notice the
    disconnect (is_alive() will return True) until we've made a failed recv()
    call. With a file socket, however, we'll also fail when calling send().
    """

    with test.runner.get_runner().get_tor_socket() as control_socket:
      control_socket.send('QUIT')
      self.assertEqual('closing connection', str(control_socket.recv()))
      self.assertTrue(control_socket.is_alive())

      # If we send another message to a port based socket then it will seem to
      # succeed. However, a file based socket should report a failure.

      if isinstance(control_socket, stem.socket.ControlPort):
        control_socket.send('blarg')
        self.assertTrue(control_socket.is_alive())
      else:
        self.assertRaises(stem.SocketClosed, control_socket.send, 'blarg')
        self.assertFalse(control_socket.is_alive())

  @test.require.controller
  def test_recv_closed(self):
    """
    Receives a message after we've closed the connection.
    """

    with test.runner.get_runner().get_tor_socket() as control_socket:
      self.assertTrue(control_socket.is_alive())
      control_socket.close()
      self.assertFalse(control_socket.is_alive())

      self.assertRaises(stem.SocketClosed, control_socket.recv)

  @test.require.controller
  def test_recv_disconnected(self):
    """
    Receives a message from a socket that has been disconnected by the other
    end.
    """

    with test.runner.get_runner().get_tor_socket() as control_socket:
      control_socket.send('QUIT')
      self.assertEqual('closing connection', str(control_socket.recv()))

      # Neither a port or file based socket will know that tor has hung up on
      # the connection at this point. We should know after calling recv(),
      # however.

      self.assertTrue(control_socket.is_alive())
      self.assertRaises(stem.SocketClosed, control_socket.recv)
      self.assertFalse(control_socket.is_alive())

  @test.require.controller
  def test_connect_repeatedly(self):
    """
    Checks that we can reconnect, use, and disconnect a socket repeatedly.
    """

    with test.runner.get_runner().get_tor_socket(False) as control_socket:
      for _ in range(10):
        # this will raise if the PROTOCOLINFO query fails
        stem.connection.get_protocolinfo(control_socket)

        control_socket.close()
        self.assertRaises(stem.SocketClosed, control_socket.send, 'PROTOCOLINFO 1')
        control_socket.connect()



"""
Tests that the stem.version functions can handle the tor instance we're
running with.
"""

import unittest

import stem.prereq
import stem.version
import test.require
import test.runner


class TestVersion(unittest.TestCase):
  @test.require.command('tor')
  def test_get_system_tor_version(self):
    """
    Basic verification checks for the get_system_tor_version() function.
    """

    # Since tor is in our path we should expect to be able to get the version
    # that way, though this might not belong to our test instance (if we're
    # running against a specific tor binary).

    stem.version.get_system_tor_version()

    # try running against a command that exists, but isn't tor
    self.assertRaises(IOError, stem.version.get_system_tor_version, 'ls')

    # try running against a command that doesn't exist
    self.assertRaises(IOError, stem.version.get_system_tor_version, 'blarg')

  @test.require.controller
  def test_getinfo_version_parsing(self):
    """
    Issues a 'GETINFO version' query to our test instance and makes sure that
    we can parse it.
    """

    control_socket = test.runner.get_runner().get_tor_socket()
    control_socket.send('GETINFO version')
    version_response = control_socket.recv()
    control_socket.close()

    # the getinfo response looks like...
    # 250-version=0.2.3.10-alpha-dev (git-65420e4cb5edcd02)
    # 250 OK

    tor_version = list(version_response)[0]
    tor_version = tor_version[8:].split(' ', 1)[0]
    stem.version.Version(tor_version)



"""
Tests installation of our library.
"""

import glob
import os
import shutil
import sys
import tarfile
import time
import unittest

import stem
import stem.prereq
import stem.util.system
import stem.util.test_tools
import test

from stem.util.test_tools import asynchronous

BASE_INSTALL_PATH = '/tmp/stem_test'
DIST_PATH = os.path.join(test.STEM_BASE, 'dist')
PYTHON_EXE = sys.executable if sys.executable else 'python'
INSTALL_MISMATCH_MSG = "Running 'python setup.py sdist' doesn't match our git contents in the following way. The manifest in our setup.py may need to be updated...\n\n"


def _assert_has_all_files(path):
  """
  Check that all the files in the stem directory are present in the
  installation. This is a very common gotcha since our setup.py
  requires us to remember to add new modules and non-source files.

  :raises: **AssertionError** files don't match our content
  """

  expected, installed = set(), set()

  for root, dirnames, filenames in os.walk(os.path.join(test.STEM_BASE, 'stem')):
    for filename in filenames:
      file_format = filename.split('.')[-1]

      if file_format not in test.IGNORED_FILE_TYPES:
        expected.add(os.path.join(root, filename)[len(test.STEM_BASE) + 1:])

  for root, dirnames, filenames in os.walk(path):
    for filename in filenames:
      if not filename.endswith('.pyc') and not filename.endswith('egg-info'):
        installed.add(os.path.join(root, filename)[len(path) + 1:])

  missing = expected.difference(installed)
  extra = installed.difference(expected)

  if missing:
    raise AssertionError("The following files were expected to be in our installation but weren't. Maybe our setup.py needs to be updated?\n\n%s" % '\n'.join(missing))
  elif extra:
    raise AssertionError("The following files weren't expected to be in our installation.\n\n%s" % '\n'.join(extra))


class TestInstallation(unittest.TestCase):
  @staticmethod
  def run_tests(args):
    test_install = stem.util.test_tools.ASYNC_TESTS['test.integ.installation.test_install']
    test_install.run()
    stem.util.test_tools.ASYNC_TESTS['test.integ.installation.test_sdist'].run(test_install.pid())

  @asynchronous
  def test_install():
    """
    Installs with 'python setup.py install' and checks we can use what we
    install.
    """

    try:
      try:
        stem.util.system.call('%s setup.py install --prefix %s' % (PYTHON_EXE, BASE_INSTALL_PATH), timeout = 60, cwd = test.STEM_BASE)
        stem.util.system.call('%s setup.py clean --all' % PYTHON_EXE, timeout = 60, cwd = test.STEM_BASE)  # tidy up the build directory

        if stem.prereq.is_pypy():
          site_packages_paths = glob.glob('%s/site-packages' % BASE_INSTALL_PATH)
        else:
          site_packages_paths = glob.glob('%s/lib*/*/site-packages' % BASE_INSTALL_PATH)
      except Exception as exc:
        raise AssertionError("Unable to install with 'python setup.py install': %s" % exc)

      if len(site_packages_paths) != 1:
        raise AssertionError('We should only have a single site-packages directory, but instead had: %s' % site_packages_paths)

      install_path = site_packages_paths[0]
      version_output = stem.util.system.call([PYTHON_EXE, '-c', "import sys;sys.path.insert(0, '%s');import stem;print(stem.__version__)" % install_path])[0]

      if stem.__version__ != version_output:
        raise AssertionError('We expected the installed version to be %s but was %s' % (stem.__version__, version_output))

      _assert_has_all_files(install_path)
    finally:
      if os.path.exists(BASE_INSTALL_PATH):
        shutil.rmtree(BASE_INSTALL_PATH)

  @asynchronous
  def test_sdist(dependency_pid):
    """
    Creates a source distribution tarball with 'python setup.py sdist' and
    checks that it matches the content of our git repository. This primarily is
    meant to test that our MANIFEST.in is up to date.
    """

    while stem.util.system.is_running(dependency_pid):
      time.sleep(0.1)  # we need to run these tests serially

    git_dir = os.path.join(test.STEM_BASE, '.git')

    if not stem.util.system.is_available('git'):
      raise stem.util.test_tools.SkipTest('(git unavailable)')
    elif not os.path.exists(git_dir):
      raise stem.util.test_tools.SkipTest('(not a git checkout)')

    if os.path.exists(DIST_PATH):
      raise AssertionError("%s already exists, maybe you manually ran 'python setup.py sdist'?" % DIST_PATH)

    try:
      try:
        stem.util.system.call('%s setup.py sdist --dryrun' % PYTHON_EXE, timeout = 60, cwd = test.STEM_BASE)
      except Exception as exc:
        raise AssertionError("Unable to run 'python setup.py sdist': %s" % exc)

      git_contents = [line.split()[-1] for line in stem.util.system.call('git --git-dir=%s ls-tree --full-tree -r HEAD' % git_dir)]

      # tarball has a prefix 'stem-[verion]' directory so stipping that out

      dist_tar = tarfile.open(os.path.join(DIST_PATH, 'stem-dry-run-%s.tar.gz' % stem.__version__))
      tar_contents = ['/'.join(info.name.split('/')[1:]) for info in dist_tar.getmembers() if info.isfile()]

      issues = []

      for path in git_contents:
        if path not in tar_contents and path not in ['.gitignore', '.travis.yml']:
          issues.append('  * %s is missing from our release tarball' % path)

      for path in tar_contents:
        if path not in git_contents and path not in ['MANIFEST.in', 'PKG-INFO']:
          issues.append("  * %s isn't expected in our release tarball" % path)

      if issues:
        raise AssertionError(INSTALL_MISMATCH_MSG + '\n'.join(issues))
    finally:
      if os.path.exists(DIST_PATH):
        shutil.rmtree(DIST_PATH)



"""
Integration tests for stem.descriptor.server_descriptor.
"""

import os
import unittest

import stem.descriptor
import stem.util.test_tools
import test

from stem.util.test_tools import asynchronous


class TestServerDescriptor(unittest.TestCase):
  @staticmethod
  def run_tests(args):
    stem.util.test_tools.ASYNC_TESTS['test.integ.descriptor.server_descriptor.test_cached_descriptor'].run(args.test_dir, threaded = True)

  @asynchronous
  def test_cached_descriptor(test_dir):
    """
    Parses the cached descriptor file in our data directory, checking that it
    doesn't raise any validation issues and looking for unrecognized descriptor
    additions.
    """

    descriptor_path = os.path.join(test_dir, 'cached-descriptors')

    if not os.path.exists(descriptor_path):
      raise stem.util.test_tools.SkipTest('(no cached descriptors)')

    with open(descriptor_path, 'rb') as descriptor_file:
      for desc in stem.descriptor.parse_file(descriptor_file, 'server-descriptor 1.0', validate = True):
        # the following attributes should be deprecated, and not appear in the wild

        if desc.read_history_end or desc.write_history_end or desc.eventdns or desc.socks_port:
          raise AssertionError('deprecated attribute appeared on: %s' % desc)

        for line in desc.get_unrecognized_lines():
          test.register_new_capability('Server Descriptor Line', line)



import os
import shutil

from stem.control import Controller
from flask import Flask

app = Flask(__name__)


@app.route('/')
def index():
  return "<h1>Hi Grandma!</h1>"


print(' * Connecting to tor')

with Controller.from_port() as controller:
  controller.authenticate()

  # All hidden services have a directory on disk. Lets put ours in tor's data
  # directory.

  hidden_service_dir = os.path.join(controller.get_conf('DataDirectory', '/tmp'), 'hello_world')

  # Create a hidden service where visitors of port 80 get redirected to local
  # port 5000 (this is where Flask runs by default).

  print(" * Creating our hidden service in %s" % hidden_service_dir)
  result = controller.create_hidden_service(hidden_service_dir, 80, target_port = 5000)

  # The hostname is only available when we can read the hidden service
  # directory. This requires us to be running with the same user as tor.

  if result.hostname:
    print(" * Our service is available at %s, press ctrl+c to quit" % result.hostname)
  else:
    print(" * Unable to determine our service's hostname, probably due to being unable to read the hidden service directory")

  try:
    app.run()
  finally:
    # Shut down the hidden service and clean it off disk. Note that you *don't*
    # want to delete the hidden service directory if you'd like to have this
    # same *.onion address in the future.

    print(" * Shutting down our hidden service")
    controller.remove_hidden_service(hidden_service_dir)
    shutil.rmtree(hidden_service_dir)



import functools

from stem import StreamStatus
from stem.control import EventType, Controller

def main():
  print("Tracking requests for tor exits. Press 'enter' to end.")
  print("")

  with Controller.from_port() as controller:
    controller.authenticate()

    stream_listener = functools.partial(stream_event, controller)
    controller.add_event_listener(stream_listener, EventType.STREAM)

    raw_input()  # wait for user to press enter


def stream_event(controller, event):
  if event.status == StreamStatus.SUCCEEDED and event.circ_id:
    circ = controller.get_circuit(event.circ_id)

    exit_fingerprint = circ.path[-1][0]
    exit_relay = controller.get_network_status(exit_fingerprint)

    print("Exit relay for our connection to %s" % (event.target))
    print("  address: %s:%i" % (exit_relay.address, exit_relay.or_port))
    print("  fingerprint: %s" % exit_relay.fingerprint)
    print("  nickname: %s" % exit_relay.nickname)
    print("  locale: %s" % controller.get_info("ip-to-country/%s" % exit_relay.address, 'unknown'))
    print("")


if __name__ == '__main__':
  main()



"""
Integration tests for stem.directory.Authority.
"""

import unittest

import stem.directory
import test.require


class TestAuthority(unittest.TestCase):
  @test.require.online
  def test_cache_is_up_to_date(self):
    """
    Check if the cached authorities we bundle are up to date.
    """

    self.assertEqual(stem.directory.Authority.from_cache(), stem.directory.Authority.from_remote())



"""
Tests for the examples given in stem's tutorial.
"""

import itertools
import os
import unittest

try:
  from StringIO import StringIO
except ImportError:
  from io import StringIO

import stem.response
import stem.descriptor.remote
import stem.prereq

from stem.control import Controller
from stem.descriptor.networkstatus import NetworkStatusDocumentV3
from stem.descriptor.router_status_entry import RouterStatusEntryV3
from stem.descriptor.server_descriptor import RelayDescriptor
from stem.directory import DIRECTORY_AUTHORITIES
from stem.response import ControlMessage

from test.unit import exec_documentation_example

try:
  # added in python 3.3
  from unittest.mock import Mock, patch
except ImportError:
  from mock import Mock, patch

OPEN_FUNCTION = open  # make a reference so mocking open() won't mess with us

CIRC_CONTENT = '650 CIRC %d %s \
%s \
PURPOSE=%s'

PATH_CONTENT = '$%s=%s,$%s=%s,$%s=%s'

LIST_CIRCUITS_OUTPUT = """\

Circuit 4 (GENERAL)
 |- B1FA7D51B8B6F0CB585D944F450E7C06EDE7E44C (ByTORAndTheSnowDog, 173.209.180.61)
 |- 0DD9935C5E939CFA1E07B8DDA6D91C1A2A9D9338 (afo02, 87.238.194.176)
 +- DB3B1CFBD3E4D97B84B548ADD5B9A31451EEC4CC (edwardsnowden3, 109.163.234.10)

Circuit 6 (GENERAL)
 |- B1FA7D51B8B6F0CB585D944F450E7C06EDE7E44C (ByTORAndTheSnowDog, 173.209.180.61)
 |- EC01CB4766BADC1611678555CE793F2A7EB2D723 (sprockets, 46.165.197.96)
 +- 9EA317EECA56BDF30CAEB208A253FB456EDAB1A0 (bolobolo1, 96.47.226.20)

Circuit 10 (GENERAL)
 |- B1FA7D51B8B6F0CB585D944F450E7C06EDE7E44C (ByTORAndTheSnowDog, 173.209.180.61)
 |- 00C2C2A16AEDB51D5E5FB7D6168FC66B343D822F (ph3x, 86.59.119.83)
 +- 65242C91BFF30F165DA4D132C81A9EBA94B71D62 (torexit16, 176.67.169.171)
"""

EXIT_USED_OUTPUT = """\
Tracking requests for tor exits. Press 'enter' to end.

Exit relay for our connection to 64.15.112.44:80
  address: 31.172.30.2:443
  fingerprint: A59E1E7C7EAEE083D756EE1FF6EC31CA3D8651D7
  nickname: chaoscomputerclub19
  locale: unknown

"""

OUTDATED_RELAYS_OUTPUT = """\
Checking for outdated relays...

  0.1.0           Sambuddha Basu

2 outdated relays found, 1 had contact information
"""

COMPARE_FLAGS_OUTPUT = """\
maatuska has the Running flag but moria1 doesn't: E2BB13AA2F6960CD93ABE5257A825687F3973C62
moria1 has the Running flag but maatuska doesn't: 546C54E2A89D88E0794D04AECBF1AC8AC9DA81DE
maatuska has the Running flag but moria1 doesn't: 92FCB6748A40E6088E22FBAB943AB2DD743EA818
maatuska has the Running flag but moria1 doesn't: 6871F682350BA931838C0EC1E4A23044DAE06A73
moria1 has the Running flag but maatuska doesn't: DCAEC3D069DC39AAE43D13C8AF31B5645E05ED61
"""

VOTES_BY_BANDWIDTH_AUTHORITIES_OUTPUT = """\
Getting gabelmoo's vote from http://131.188.40.189:80/tor/status-vote/current/authority:
  5935 measured entries and 1332 unmeasured
Getting moria1's vote from http://128.31.0.39:9131/tor/status-vote/current/authority:
  6647 measured entries and 625 unmeasured
Getting maatuska's vote from http://171.25.193.9:443/tor/status-vote/current/authority:
  6313 measured entries and 1112 unmeasured
"""

PERSISTING_A_CONSENSUS_OUTPUT = """\
A7569A83B5706AB1B1A9CB52EFF7D2D32E4553EB: caerSidi
"""


def _get_event(content):
  return ControlMessage.from_str(content, 'EVENT', normalize = True)


def _get_circ_event(id, status, hop1, hop2, hop3, purpose):
  path = PATH_CONTENT % (hop1[0], hop1[1], hop2[0], hop2[1], hop3[0], hop3[1])
  content = CIRC_CONTENT % (id, status, path, purpose)
  return _get_event(content)


def _get_router_status(address = None, port = None, nickname = None, fingerprint_base64 = None, s_line = None):
  r_line = 'caerSidi p1aag7VwarGxqctS7/fS0y5FU+s oQZFLYe9e4A7bOkWKR7TaNxb0JE 2012-08-06 11:19:31 71.35.150.29 9001 0'

  if address:
    r_line = r_line.replace('71.35.150.29', address)

  if port:
    r_line = r_line.replace('9001', port)

  if nickname:
    r_line = r_line.replace('caerSidi', nickname)

  if fingerprint_base64:
    r_line = r_line.replace('p1aag7VwarGxqctS7/fS0y5FU+s', fingerprint_base64)

  if s_line:
    return RouterStatusEntryV3.create({'r': r_line, 's': s_line})
  else:
    return RouterStatusEntryV3.create({'r': r_line})


class TestTutorialExamples(unittest.TestCase):
  def assert_equal_unordered(self, expected, actual):
    if stem.prereq.is_python_3():
      self.assertCountEqual(expected.splitlines(), actual.splitlines())
    else:
      self.assertItemsEqual(expected.splitlines(), actual.splitlines())

  @patch('sys.stdout', new_callable = StringIO)
  @patch('stem.control.Controller.from_port', spec = Controller)
  def test_list_circuits(self, from_port_mock, stdout_mock):
    path_1 = ('B1FA7D51B8B6F0CB585D944F450E7C06EDE7E44C', 'ByTORAndTheSnowDog')
    path_2 = ('0DD9935C5E939CFA1E07B8DDA6D91C1A2A9D9338', 'afo02')
    path_3 = ('DB3B1CFBD3E4D97B84B548ADD5B9A31451EEC4CC', 'edwardsnowden3')
    path_4 = ('EC01CB4766BADC1611678555CE793F2A7EB2D723', 'sprockets')
    path_5 = ('9EA317EECA56BDF30CAEB208A253FB456EDAB1A0', 'bolobolo1')
    path_6 = ('00C2C2A16AEDB51D5E5FB7D6168FC66B343D822F', 'ph3x')
    path_7 = ('65242C91BFF30F165DA4D132C81A9EBA94B71D62', 'torexit16')

    circuit_4 = _get_circ_event(4, 'BUILT', path_1, path_2, path_3, 'GENERAL')
    circuit_6 = _get_circ_event(6, 'BUILT', path_1, path_4, path_5, 'GENERAL')
    circuit_10 = _get_circ_event(10, 'BUILT', path_1, path_6, path_7, 'GENERAL')

    controller = from_port_mock().__enter__()
    controller.get_circuits.return_value = [circuit_4, circuit_6, circuit_10]

    controller.get_network_status.side_effect = lambda fingerprint, *args: {
      path_1[0]: _get_router_status('173.209.180.61'),
      path_2[0]: _get_router_status('87.238.194.176'),
      path_3[0]: _get_router_status('109.163.234.10'),
      path_4[0]: _get_router_status('46.165.197.96'),
      path_5[0]: _get_router_status('96.47.226.20'),
      path_6[0]: _get_router_status('86.59.119.83'),
      path_7[0]: _get_router_status('176.67.169.171')
    }[fingerprint]

    exec_documentation_example('list_circuits.py')
    self.assert_equal_unordered(LIST_CIRCUITS_OUTPUT, stdout_mock.getvalue())

  @patch('sys.stdout', new_callable = StringIO)
  @patch('stem.control.Controller.from_port', spec = Controller)
  def test_exit_used(self, from_port_mock, stdout_mock):
    def tutorial_example(mock_event):
      import functools

      from stem import StreamStatus
      from stem.control import EventType, Controller

      def main():
        print("Tracking requests for tor exits. Press 'enter' to end.\n")

        with Controller.from_port() as controller:
          controller.authenticate()

          stream_listener = functools.partial(stream_event, controller)
          controller.add_event_listener(stream_listener, EventType.STREAM)

          stream_event(controller, mock_event)  # simulate an event during the raw_input()

      def stream_event(controller, event):
        if event.status == StreamStatus.SUCCEEDED and event.circ_id:
          circ = controller.get_circuit(event.circ_id)

          exit_fingerprint = circ.path[-1][0]
          exit_relay = controller.get_network_status(exit_fingerprint)

          print('Exit relay for our connection to %s' % (event.target))
          print('  address: %s:%i' % (exit_relay.address, exit_relay.or_port))
          print('  fingerprint: %s' % exit_relay.fingerprint)
          print('  nickname: %s' % exit_relay.nickname)
          print('  locale: %s\n' % controller.get_info('ip-to-country/%s' % exit_relay.address, 'unknown'))

      main()

    path_1 = ('9EA317EECA56BDF30CAEB208A253FB456EDAB1A0', 'bolobolo1')
    path_2 = ('00C2C2A16AEDB51D5E5FB7D6168FC66B343D822F', 'ph3x')
    path_3 = ('A59E1E7C7EAEE083D756EE1FF6EC31CA3D8651D7', 'chaoscomputerclub19')
    circuit = _get_circ_event(1, 'BUILT', path_1, path_2, path_3, 'GENERAL')

    event_content = '650 STREAM 15 SUCCEEDED 3 64.15.112.44:80'
    event = _get_event(event_content)

    controller = from_port_mock().__enter__()
    controller.get_circuit.return_value = circuit
    controller.get_network_status.return_value = _get_router_status('31.172.30.2', '443', path_3[1], 'pZ4efH6u4IPXVu4f9uwxyj2GUdc=')
    controller.get_info.return_value = 'unknown'

    tutorial_example(event)
    self.assert_equal_unordered(EXIT_USED_OUTPUT, stdout_mock.getvalue())

  @patch('sys.stdout', new_callable = StringIO)
  @patch('stem.descriptor.remote.DescriptorDownloader')
  def test_outdated_relays(self, downloader_mock, stdout_mock):
    downloader_mock().get_server_descriptors.return_value = [
      RelayDescriptor.create({'platform': 'node-Tor 0.2.3.0 on Linux x86_64'}),
      RelayDescriptor.create({'platform': 'node-Tor 0.1.0 on Linux x86_64'}),
      RelayDescriptor.create({'opt': 'contact Random Person admin@gtr-10.de', 'platform': 'node-Tor 0.2.3.0 on Linux x86_64'}),
      RelayDescriptor.create({'opt': 'contact Sambuddha Basu', 'platform': 'node-Tor 0.1.0 on Linux x86_64'}),
    ]

    exec_documentation_example('outdated_relays.py')

    self.assert_equal_unordered(OUTDATED_RELAYS_OUTPUT, stdout_mock.getvalue())

  @patch('sys.stdout', new_callable = StringIO)
  @patch('stem.descriptor.remote.Query')
  @patch('stem.directory.Authority.from_cache')
  def test_compare_flags(self, authorities_mock, query_mock, stdout_mock):
    if stem.prereq._is_python_26():
      # example imports OrderedDict from collections which doesn't work under
      # python 2.6

      self.skipTest("(example doesn't support python 2.6)")
      return

    authorities_mock().items.return_value = [('moria1', DIRECTORY_AUTHORITIES['moria1']), ('maatuska', DIRECTORY_AUTHORITIES['maatuska'])]

    fingerprint = [
      ('92FCB6748A40E6088E22FBAB943AB2DD743EA818', 'kvy2dIpA5giOIvurlDqy3XQ+qBg='),
      ('6871F682350BA931838C0EC1E4A23044DAE06A73', 'aHH2gjULqTGDjA7B5KIwRNrganM='),
      ('E2BB13AA2F6960CD93ABE5257A825687F3973C62', '4rsTqi9pYM2Tq+UleoJWh/OXPGI='),
      ('546C54E2A89D88E0794D04AECBF1AC8AC9DA81DE', 'VGxU4qidiOB5TQSuy/Gsisnagd4='),
      ('DCAEC3D069DC39AAE43D13C8AF31B5645E05ED61', '3K7D0GncOarkPRPIrzG1ZF4F7WE='),
    ]

    entry = [
      # entries for moria1

      _get_router_status(fingerprint_base64 = fingerprint[0][1], s_line = ' '),
      _get_router_status(fingerprint_base64 = fingerprint[1][1], s_line = ' '),
      _get_router_status(fingerprint_base64 = fingerprint[2][1], s_line = ' '),
      _get_router_status(fingerprint_base64 = fingerprint[3][1]),
      _get_router_status(fingerprint_base64 = fingerprint[4][1]),

      # entries for maatuska

      _get_router_status(fingerprint_base64 = fingerprint[0][1]),
      _get_router_status(fingerprint_base64 = fingerprint[1][1]),
      _get_router_status(fingerprint_base64 = fingerprint[2][1]),
      _get_router_status(fingerprint_base64 = fingerprint[3][1], s_line = ' '),
      _get_router_status(fingerprint_base64 = fingerprint[4][1], s_line = ' '),
    ]

    query_mock().run.side_effect = [
      [NetworkStatusDocumentV3.create(routers = (entry[0], entry[1], entry[2], entry[3], entry[4]))],
      [NetworkStatusDocumentV3.create(routers = (entry[5], entry[6], entry[7], entry[8], entry[9]))],
    ]

    exec_documentation_example('compare_flags.py')

    self.assert_equal_unordered(COMPARE_FLAGS_OUTPUT, stdout_mock.getvalue())

  @patch('sys.stdout', new_callable = StringIO)
  @patch('stem.directory.Authority.from_cache')
  @patch('stem.descriptor.remote.DescriptorDownloader.query')
  def test_votes_by_bandwidth_authorities(self, query_mock, authorities_mock, stdout_mock):
    directory_values = [
      DIRECTORY_AUTHORITIES['gabelmoo'],
      DIRECTORY_AUTHORITIES['moria1'],
      DIRECTORY_AUTHORITIES['maatuska'],
    ]

    directory_values[0].address = '131.188.40.189'
    authorities_mock().values.return_value = directory_values

    entry_with_measurement = RouterStatusEntryV3.create({'w': 'Bandwidth=1 Measured=1'})
    entry_without_measurement = RouterStatusEntryV3.create()

    query1 = Mock()
    query1.download_url = 'http://131.188.40.189:80/tor/status-vote/current/authority'
    query1.run.return_value = [entry_with_measurement] * 5935 + [entry_without_measurement] * 1332

    query2 = Mock()
    query2.download_url = 'http://128.31.0.39:9131/tor/status-vote/current/authority'
    query2.run.return_value = [entry_with_measurement] * 6647 + [entry_without_measurement] * 625

    query3 = Mock()
    query3.download_url = 'http://171.25.193.9:443/tor/status-vote/current/authority'
    query3.run.return_value = [entry_with_measurement] * 6313 + [entry_without_measurement] * 1112

    query_mock.side_effect = [query1, query2, query3]

    exec_documentation_example('votes_by_bandwidth_authorities.py')
    self.assert_equal_unordered(VOTES_BY_BANDWIDTH_AUTHORITIES_OUTPUT, stdout_mock.getvalue())

  @patch('sys.stdout', new_callable = StringIO)
  @patch('stem.descriptor.parse_file')
  @patch('stem.descriptor.remote.Query')
  def test_persisting_a_consensus(self, query_mock, parse_file_mock, stdout_mock):
    def tutorial_example_2():
      from stem.descriptor import DocumentHandler, parse_file

      consensus = next(parse_file(
        '/tmp/descriptor_dump',
        descriptor_type = 'network-status-consensus-3 1.0',
        document_handler = DocumentHandler.DOCUMENT,
      ))

      for fingerprint, relay in consensus.routers.items():
        print('%s: %s' % (fingerprint, relay.nickname))

    network_status = NetworkStatusDocumentV3.create(routers = (RouterStatusEntryV3.create({
      'r': 'caerSidi p1aag7VwarGxqctS7/fS0y5FU+s oQZFLYe9e4A7bOkWKR7TaNxb0JE 2012-08-06 11:19:31 71.35.150.29 9001 0',
    }),))

    query_mock().run.return_value = [network_status]
    parse_file_mock.return_value = itertools.cycle([network_status])

    exec_documentation_example('persisting_a_consensus.py')
    exec_documentation_example('persisting_a_consensus_with_parse_file.py')

    self.assertEqual(PERSISTING_A_CONSENSUS_OUTPUT, stdout_mock.getvalue())

    if os.path.exists('/tmp/descriptor_dump'):
      os.remove('/tmp/descriptor_dump')



"""
Unit tests for stem.descriptor.export.
"""

import unittest

try:
  from StringIO import StringIO
except ImportError:
  from io import StringIO

import stem.prereq

from stem.descriptor.server_descriptor import RelayDescriptor, BridgeDescriptor
from stem.descriptor.export import export_csv, export_csv_file


class TestExport(unittest.TestCase):
  def test_minimal_descriptor(self):
    """
    Exports a single minimal tor server descriptor.
    """

    if stem.prereq._is_python_26():
      self.skipTest('(header added in python 2.7)')
      return

    desc = RelayDescriptor.create({
      'router': 'caerSidi 71.35.133.197 9001 0 0',
      'published': '2012-03-01 17:15:27',
    })

    desc_csv = export_csv(desc, included_fields = ('nickname', 'address', 'published'), header = False)
    expected = 'caerSidi,71.35.133.197,2012-03-01 17:15:27\n'
    self.assertEqual(expected, desc_csv)

    desc_csv = export_csv(desc, included_fields = ('nickname', 'address', 'published'), header = True)
    expected = 'nickname,address,published\n' + expected
    self.assertEqual(expected, desc_csv)

  def test_multiple_descriptors(self):
    """
    Exports multiple descriptors, making sure that we get them back in the same
    order.
    """

    nicknames = ('relay1', 'relay3', 'relay2', 'caerSidi', 'zeus')
    descriptors = []

    for nickname in nicknames:
      router_line = '%s 71.35.133.197 9001 0 0' % nickname
      descriptors.append(RelayDescriptor.create({'router': router_line}))

    expected = '\n'.join(nicknames) + '\n'
    self.assertEqual(expected, export_csv(descriptors, included_fields = ('nickname',), header = False))

  def test_file_output(self):
    """
    Basic test for the export_csv_file() function, checking that it provides
    the same output as export_csv().
    """

    desc = RelayDescriptor.create()
    desc_csv = export_csv(desc)

    csv_buffer = StringIO()
    export_csv_file(csv_buffer, desc)

    self.assertEqual(desc_csv, csv_buffer.getvalue())

  def test_excludes_private_attr(self):
    """
    Checks that the default attributes for our csv output doesn't include private fields.
    """

    if stem.prereq._is_python_26():
      self.skipTest('(header added in python 2.7)')
      return

    desc = RelayDescriptor.create()
    desc_csv = export_csv(desc)

    self.assertTrue(',signature' in desc_csv)
    self.assertFalse(',_digest' in desc_csv)
    self.assertFalse(',_annotation_lines' in desc_csv)

  def test_empty_input(self):
    """
    Exercises when we don't provide any descriptors.
    """
    self.assertEqual('', export_csv([]))

  def test_invalid_attributes(self):
    """
    Attempts to make a csv with attributes that don't exist.
    """

    desc = RelayDescriptor.create()
    self.assertRaises(ValueError, export_csv, desc, ('nickname', 'blarg!'))

  def test_multiple_descriptor_types(self):
    """
    Attempts to make a csv with multiple descriptor types.
    """

    self.assertRaises(ValueError, export_csv, (RelayDescriptor.create(), BridgeDescriptor.create()))



"""
Unit tests for the stem.response.ControlLine class.
"""

import unittest

import stem.response

# response made by having 'DataDirectory /tmp/my data\"dir/' in the torrc
PROTOCOLINFO_RESPONSE = (
  'PROTOCOLINFO 1',
  'AUTH METHODS=COOKIE COOKIEFILE="/tmp/my data\\\\\\"dir//control_auth_cookie"',
  'VERSION Tor="0.2.1.30"',
  'OK',
)


class TestControlLine(unittest.TestCase):
  def test_pop_examples(self):
    """
    Checks that the pop method's pydoc examples are correct.
    """

    line = stem.response.ControlLine("\"We're all mad here.\" says the grinning cat.")
    self.assertEqual(line.pop(True), "We're all mad here.")
    self.assertEqual(line.pop(), 'says')
    self.assertEqual(line.remainder(), 'the grinning cat.')

    line = stem.response.ControlLine('"this has a \\" and \\\\ in it" foo=bar more_data')
    self.assertEqual(line.pop(True, True), 'this has a " and \\ in it')

  def test_string(self):
    """
    Basic checks that we behave as a regular immutable string.
    """

    line = stem.response.ControlLine(PROTOCOLINFO_RESPONSE[0])
    self.assertEqual(line, 'PROTOCOLINFO 1')
    self.assertTrue(line.startswith('PROTOCOLINFO '))

    # checks that popping items doesn't effect us
    line.pop()
    self.assertEqual(line, 'PROTOCOLINFO 1')
    self.assertTrue(line.startswith('PROTOCOLINFO '))

  def test_general_usage(self):
    """
    Checks a basic use case for the popping entries.
    """

    # pops a series of basic, space separated entries
    line = stem.response.ControlLine(PROTOCOLINFO_RESPONSE[0])
    self.assertEqual(line.remainder(), 'PROTOCOLINFO 1')
    self.assertFalse(line.is_empty())
    self.assertFalse(line.is_next_quoted())
    self.assertFalse(line.is_next_mapping())
    self.assertEqual(None, line.peek_key())

    self.assertRaises(ValueError, line.pop_mapping)
    self.assertEqual(line.pop(), 'PROTOCOLINFO')
    self.assertEqual(line.remainder(), '1')
    self.assertFalse(line.is_empty())
    self.assertFalse(line.is_next_quoted())
    self.assertFalse(line.is_next_mapping())
    self.assertEqual(None, line.peek_key())

    self.assertRaises(ValueError, line.pop_mapping)
    self.assertEqual(line.pop(), '1')
    self.assertEqual(line.remainder(), '')
    self.assertTrue(line.is_empty())
    self.assertFalse(line.is_next_quoted())
    self.assertFalse(line.is_next_mapping())
    self.assertEqual(None, line.peek_key())

    self.assertRaises(IndexError, line.pop_mapping)
    self.assertRaises(IndexError, line.pop)
    self.assertEqual(line.remainder(), '')
    self.assertTrue(line.is_empty())
    self.assertFalse(line.is_next_quoted())
    self.assertFalse(line.is_next_mapping())
    self.assertEqual(None, line.peek_key())

  def test_pop_mapping(self):
    """
    Checks use cases when parsing KEY=VALUE mappings.
    """

    # version entry with a space
    version_entry = 'Tor="0.2.1.30 (0a083b0188cacd2f07838ff0446113bd5211a024)"'

    line = stem.response.ControlLine(version_entry)
    self.assertEqual(line.remainder(), version_entry)
    self.assertFalse(line.is_empty())
    self.assertFalse(line.is_next_quoted())
    self.assertTrue(line.is_next_mapping())
    self.assertTrue(line.is_next_mapping(key = 'Tor'))
    self.assertTrue(line.is_next_mapping(key = 'Tor', quoted = True))
    self.assertTrue(line.is_next_mapping(quoted = True))
    self.assertEqual('Tor', line.peek_key())

    # try popping this as a non-quoted mapping
    self.assertEqual(line.pop_mapping(), ('Tor', '"0.2.1.30'))
    self.assertEqual(line.remainder(), '(0a083b0188cacd2f07838ff0446113bd5211a024)"')
    self.assertFalse(line.is_empty())
    self.assertFalse(line.is_next_quoted())
    self.assertFalse(line.is_next_mapping())
    self.assertRaises(ValueError, line.pop_mapping)
    self.assertEqual(None, line.peek_key())

    # try popping this as a quoted mapping
    line = stem.response.ControlLine(version_entry)
    self.assertEqual(line.pop_mapping(True), ('Tor', '0.2.1.30 (0a083b0188cacd2f07838ff0446113bd5211a024)'))
    self.assertEqual(line.remainder(), '')
    self.assertTrue(line.is_empty())
    self.assertFalse(line.is_next_quoted())
    self.assertFalse(line.is_next_mapping())
    self.assertEqual(None, line.peek_key())

  def test_escapes(self):
    """
    Checks that we can parse quoted values with escaped quotes in it. This
    explicitely comes up with the COOKIEFILE attribute of PROTOCOLINFO
    responses.
    """

    auth_line = PROTOCOLINFO_RESPONSE[1]
    line = stem.response.ControlLine(auth_line)
    self.assertEqual(line, auth_line)
    self.assertEqual(line.remainder(), auth_line)

    self.assertEqual(line.pop(), 'AUTH')
    self.assertEqual(line.pop_mapping(), ('METHODS', 'COOKIE'))

    self.assertEqual(line.remainder(), r'COOKIEFILE="/tmp/my data\\\"dir//control_auth_cookie"')
    self.assertTrue(line.is_next_mapping())
    self.assertTrue(line.is_next_mapping(key = 'COOKIEFILE'))
    self.assertTrue(line.is_next_mapping(quoted = True))
    self.assertTrue(line.is_next_mapping(quoted = True, escaped = True))
    cookie_file_entry = line.remainder()

    # try a general pop
    self.assertEqual(line.pop(), 'COOKIEFILE="/tmp/my')
    self.assertEqual(line.pop(), r'data\\\"dir//control_auth_cookie"')
    self.assertTrue(line.is_empty())

    # try a general pop with escapes
    line = stem.response.ControlLine(cookie_file_entry)
    self.assertEqual(line.pop(escaped = True), 'COOKIEFILE="/tmp/my')
    self.assertEqual(line.pop(escaped = True), r'data\"dir//control_auth_cookie"')
    self.assertTrue(line.is_empty())

    # try a mapping pop
    line = stem.response.ControlLine(cookie_file_entry)
    self.assertEqual(line.pop_mapping(), ('COOKIEFILE', '"/tmp/my'))
    self.assertEqual(line.remainder(), r'data\\\"dir//control_auth_cookie"')
    self.assertFalse(line.is_empty())

    # try a quoted mapping pop (this should trip up on the escaped quote)
    line = stem.response.ControlLine(cookie_file_entry)
    self.assertEqual(line.pop_mapping(True), ('COOKIEFILE', '/tmp/my data\\\\\\'))
    self.assertEqual(line.remainder(), 'dir//control_auth_cookie"')
    self.assertFalse(line.is_empty())

    # try an escaped quoted mapping pop
    line = stem.response.ControlLine(cookie_file_entry)
    self.assertEqual(line.pop_mapping(True, True), ('COOKIEFILE', r'/tmp/my data\"dir//control_auth_cookie'))
    self.assertTrue(line.is_empty())

    # try an escaped slash followed by a character that could be part of an
    # escape sequence

    line = stem.response.ControlLine(r'COOKIEFILE="C:\\Users\\Atagar\\AppData\\tor\\control_auth_cookie"')
    self.assertEqual(line.pop_mapping(True, True), ('COOKIEFILE', r'C:\Users\Atagar\AppData\tor\control_auth_cookie'))
    self.assertTrue(line.is_empty())



import stem.descriptor.remote

server_descriptors = stem.descriptor.remote.get_server_descriptors().run()

with open('/tmp/descriptor_dump', 'wb') as descriptor_file:
  descriptor_file.write(''.join(map(str, server_descriptors)))



"""
Unit testing code for the stem.util.proc functions.
"""

import io
import unittest

import test

from stem.util import proc
from stem.util.connection import Connection

try:
  from unittest.mock import Mock, patch
except ImportError:
  from mock import Mock, patch

TITLE_LINE = b'sl  local_address rem_address   st tx_queue rx_queue tr tm->when retrnsmt  uid  timeout'

TCP6_CONTENT = b"""\
  sl  local_address                         remote_address                        st tx_queue rx_queue tr tm->when retrnsmt   uid  timeout inode
   0: 00000000000000000000000000000000:1495 00000000000000000000000000000000:0000 0A 00000000:00000000 00:00000000 00000000   106        0 14347030 1 0000000000000000 100 0 0 10 0
   1: 00000000000000000000000000000000:0035 00000000000000000000000000000000:0000 0A 00000000:00000000 00:00000000 00000000     0        0 1457 1 0000000000000000 100 0 0 10 0
   2: 00000000000000000000000000000000:0217 00000000000000000000000000000000:0000 0A 00000000:00000000 00:00000000 00000000     0        0 6606 1 0000000000000000 100 0 0 10 0
   3: F804012A4A5190010000000002000000:01BB 00000000000000000000000000000000:0000 0A 00000000:00000000 00:00000000 00000000     0        0 4372 1 0000000000000000 100 0 0 10 0
   4: 00000000000000000000000000000000:14A1 00000000000000000000000000000000:0000 0A 00000000:00000000 00:00000000 00000000   106        0 14347031 1 0000000000000000 100 0 0 10 0
   5: 00000000000000000000000000000000:1466 00000000000000000000000000000000:0000 0A 00000000:00000000 00:00000000 00000000   106        0 14347029 1 0000000000000000 100 0 0 10 0
   6: F804012A4A5190010000000002000000:01BB 38060120404100A0000000008901FFFF:9DF3 01 00000000:00000000 00:00000000 00000000   101        0 42088802 1 0000000000000000 20 4 25 10 7
   7: F804012A4A5190010000000002000000:01BB 58080120020002000000BBAA26153B56:ADB5 01 00000000:00000000 00:00000000 00000000   101        0 41691357 1 0000000000000000 24 4 32 10 7
   8: 0000000000000000FFFF00004B9E0905:1466 0000000000000000FFFF00002186364E:95BA 01 00000000:00000000 02:000A5B3D 00000000   106        0 41878761 2 0000000000000000 26 4 30 10 -1
   9: F804012A4A5190010000000002000000:1495 F806012011006F120000000026000000:C5A2 01 00000000:00000000 02:000A5B3D 00000000   106        0 41825895 2 0000000000000000 21 4 15 10 -1
  10: 0000000000000000FFFF00004B9E0905:1466 0000000000000000FFFF00002186364E:951E 01 00000000:00000000 02:00090E70 00000000   106        0 41512577 2 0000000000000000 26 4 31 10 -1
"""

SL_WIDTH_CHANGE = b"""\
  sl  local_address rem_address   st tx_queue rx_queue tr tm->when retrnsmt   uid  timeout inode
   0: 11111111:1111 22222233:2244 01 44444444:44444444 55:55555555 66666666  1111        8 99999999
   1: 22222222:2222 33333344:3355 01 44444444:44444444 55:55555555 66666666  1111        8 88888888
12675: 33333333:3333 44444455:4466 01 44444444:44444444 55:55555555 66666666  1111        8 77777777
""".rstrip()


class TestProc(unittest.TestCase):
  @patch('stem.util.proc._get_line')
  def test_system_start_time(self, get_line_mock):
    """
    Tests the system_start_time function.
    """

    get_line_mock.side_effect = lambda *params: {
      ('/proc/stat', 'btime', 'system start time'): 'btime 1001001',
    }[params]

    self.assertEqual(1001001, proc.system_start_time())

  @patch('stem.util.proc._get_line')
  def test_physical_memory(self, get_line_mock):
    """
    Tests the physical_memory function.
    """

    get_line_mock.side_effect = lambda *params: {
      ('/proc/meminfo', 'MemTotal:', 'system physical memory'): 'MemTotal:       12345 kB',
    }[params]

    self.assertEqual((12345 * 1024), proc.physical_memory())

  @patch('os.readlink')
  def test_cwd(self, readlink_mock):
    """
    Tests the cwd function with a given pid.
    """

    readlink_mock.side_effect = lambda param: {
      '/proc/24019/cwd': '/home/directory/TEST'
    }[param]

    self.assertEqual('/home/directory/TEST', proc.cwd(24019))

  @patch('stem.util.proc._get_line')
  def test_uid(self, get_line_mock):
    """
    Tests the uid function with a given pid.
    """

    for test_value in [(24019, 11111), (0, 22222)]:
      pid, uid = test_value

      get_line_mock.side_effect = lambda *params: {
        ('/proc/%s/status' % pid, 'Uid:', 'uid'): 'Uid: %s' % uid,
      }[params]

      self.assertEqual(uid, proc.uid(pid))

  @patch('stem.util.proc._get_lines')
  def test_memory_usage(self, get_lines_mock):
    """
    Tests the memory_usage function with a given pid.
    """

    get_lines_mock.side_effect = lambda *params: {
      ('/proc/1111/status', ('VmRSS:', 'VmSize:'), 'memory usage'):
        {'VmRSS:': 'VmRSS: 100 kB', 'VmSize:': 'VmSize: 1800 kB'}
    }[params]

    self.assertEqual((0, 0), proc.memory_usage(0))
    self.assertEqual((100 * 1024, 1800 * 1024), proc.memory_usage(1111))

  @patch('stem.util.proc._get_line')
  @patch('stem.util.proc.system_start_time', Mock(return_value = 10))
  @patch('stem.util.proc.CLOCK_TICKS', 100)
  def test_stats(self, get_line_mock):
    """
    Tests stats() with all combinations of stat_type arguments.
    """

    # list of all combinations of args with respective return values
    stat_combinations = test.get_all_combinations([
      ('command', 'test_program'),
      ('utime', '0.13'),
      ('stime', '0.14'),
      ('start time', '10.21'),
    ])

    stat_path = '/proc/24062/stat'
    stat = '1 (test_program) 2 3 4 5 6 7 8 9 10 11 12 13.0 14.0 15 16 17 18 19 20 21.0 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43'

    # tests the case where no stat_types are specified

    get_line_mock.side_effect = lambda *params: {
      (stat_path, '24062', 'process '): stat
    }[params]

    self.assertEqual((), proc.stats(24062))

    for stats in stat_combinations:
      # the stats variable is...
      #   [(arg1, resp1), (arg2, resp2)...]
      #
      # but we need...
      #   (arg1, arg2...), (resp1, resp2...).

      args, response = list(zip(*stats))

      get_line_mock.side_effect = lambda *params: {
        (stat_path, '24062', 'process %s' % ', '.join(args)): stat
      }[params]

      self.assertEqual(response, proc.stats(24062, *args))

      # tests the case where pid = 0

      if 'start time' in args:
        response = 10
      else:
        response = ()

        for arg in args:
          if arg == 'command':
            response += ('sched',)
          elif arg == 'utime':
            response += ('0',)
          elif arg == 'stime':
            response += ('0',)

      get_line_mock.side_effect = lambda *params: {
        ('/proc/0/stat', '0', 'process %s' % ', '.join(args)): stat
      }[params]

      self.assertEqual(response, proc.stats(0, *args))

  @patch('os.listdir')
  def test_file_descriptors_used(self, listdir_mock):
    """
    Tests the file_descriptors_used function.
    """

    # check that we reject bad pids

    for arg in (None, -100, 'hello',):
      self.assertRaises(IOError, proc.file_descriptors_used, arg)

    # when proc directory doesn't exist

    error_msg = "OSError: [Errno 2] No such file or directory: '/proc/2118/fd'"
    listdir_mock.side_effect = OSError(error_msg)

    exc_msg = 'Unable to check number of file descriptors used: %s' % error_msg
    self.assertRaisesWith(IOError, exc_msg, proc.file_descriptors_used, 2118)

    # successful calls

    listdir_mock.return_value = ['0', '1', '2', '3', '4', '5']
    listdir_mock.side_effect = None

    self.assertEqual(6, proc.file_descriptors_used(2118))
    self.assertEqual(6, proc.file_descriptors_used('2118'))

  @patch('os.listdir')
  @patch('os.path.exists')
  @patch('os.readlink')
  @patch('stem.util.proc.open', create = True)
  def test_connections(self, open_mock, readlink_mock, path_exists_mock, listdir_mock):
    """
    Tests the connections function.
    """

    pid = 1111

    listdir_mock.side_effect = lambda param: {
      '/proc/%s/fd' % pid: ['1', '2', '3', '4'],
    }[param]

    readlink_mock.side_effect = lambda param: {
      '/proc/%s/fd/1' % pid: 'socket:[99999999]',
      '/proc/%s/fd/2' % pid: 'socket:[IIIIIIII]',
      '/proc/%s/fd/3' % pid: 'pipe:[30303]',
      '/proc/%s/fd/4' % pid: 'pipe:[40404]',
    }[param]

    tcp = TITLE_LINE + b'\n 0: 11111111:1111 22222222:2222 01 44444444:44444444 55:55555555 66666666 1111        8 99999999'
    udp = TITLE_LINE + b'\n A: BBBBBBBB:BBBB CCCCCCCC:CCCC DD EEEEEEEE:EEEEEEEE FF:FFFFFFFF GGGGGGGG 1111        H IIIIIIII'

    path_exists_mock.side_effect = lambda param: {
      '/proc/net/tcp6': False,
      '/proc/net/udp6': False
    }[param]

    open_mock.side_effect = lambda param, mode: {
      '/proc/net/tcp': io.BytesIO(tcp),
      '/proc/net/udp': io.BytesIO(udp)
    }[param]

    expected = [
      Connection('17.17.17.17', 4369, '34.34.34.34', 8738, 'tcp', False),
      Connection('187.187.187.187', 48059, '204.204.204.204', 52428, 'udp', False),
    ]

    self.assertEqual(expected, proc.connections(pid))

  @patch('os.listdir')
  @patch('os.path.exists')
  @patch('os.readlink')
  @patch('stem.util.proc.open', create = True)
  def test_connections_ipv6(self, open_mock, readlink_mock, path_exists_mock, listdir_mock):
    """
    Tests the connections function with ipv6 addresses.
    """

    pid = 1111

    listdir_mock.side_effect = lambda param: {
      '/proc/%s/fd' % pid: ['1', '2'],
    }[param]

    readlink_mock.side_effect = lambda param: {
      '/proc/%s/fd/1' % pid: 'socket:[42088802]',
      '/proc/%s/fd/2' % pid: 'socket:[41691357]',
    }[param]

    path_exists_mock.side_effect = lambda param: {
      '/proc/net/tcp6': True,
      '/proc/net/udp6': False
    }[param]

    open_mock.side_effect = lambda param, mode: {
      '/proc/net/tcp': io.BytesIO(TITLE_LINE),
      '/proc/net/tcp6': io.BytesIO(TCP6_CONTENT),
      '/proc/net/udp': io.BytesIO(TITLE_LINE),
    }[param]

    expected = [
      Connection('2a01:04f8:0190:514a:0000:0000:0000:0002', 443, '2001:0638:a000:4140:0000:0000:ffff:0189', 40435, 'tcp', True),
      Connection('2a01:04f8:0190:514a:0000:0000:0000:0002', 443, '2001:0858:0002:0002:aabb:0000:563b:1526', 44469, 'tcp', True),
    ]

    self.assertEqual(expected, proc.connections(pid = pid))

  @patch('os.path.exists')
  @patch('pwd.getpwnam')
  @patch('stem.util.proc.open', create = True)
  def test_connections_ipv6_by_user(self, open_mock, getpwnam_mock, path_exists_mock):
    """
    Tests the connections function with ipv6 addresses.
    """

    getpwnam_mock('me').pw_uid = 106

    path_exists_mock.side_effect = lambda param: {
      '/proc/net/tcp6': True,
      '/proc/net/udp6': False
    }[param]

    open_mock.side_effect = lambda param, mode: {
      '/proc/net/tcp': io.BytesIO(TITLE_LINE),
      '/proc/net/tcp6': io.BytesIO(TCP6_CONTENT),
      '/proc/net/udp': io.BytesIO(TITLE_LINE),
    }[param]

    expected = [
      Connection('0000:0000:0000:0000:0000:ffff:0509:9e4b', 5222, '0000:0000:0000:0000:0000:ffff:4e36:8621', 38330, 'tcp', True),
      Connection('2a01:04f8:0190:514a:0000:0000:0000:0002', 5269, '2001:06f8:126f:0011:0000:0000:0000:0026', 50594, 'tcp', True),
      Connection('0000:0000:0000:0000:0000:ffff:0509:9e4b', 5222, '0000:0000:0000:0000:0000:ffff:4e36:8621', 38174, 'tcp', True),
    ]

    self.assertEqual(expected, proc.connections(user = 'me'))

  @patch('os.listdir')
  @patch('os.path.exists')
  @patch('os.readlink')
  @patch('stem.util.proc.open', create = True)
  def test_high_connection_count(self, open_mock, readlink_mock, path_exists_mock, listdir_mock):
    """
    When we have over ten thousand connections the 'SL' column's width changes.
    Checking that we account for this.
    """

    pid = 1111

    listdir_mock.side_effect = lambda param: {
      '/proc/%s/fd' % pid: ['1', '2', '3', '4'],
    }[param]

    readlink_mock.side_effect = lambda param: {
      '/proc/%s/fd/1' % pid: 'socket:[99999999]',
      '/proc/%s/fd/2' % pid: 'socket:[88888888]',
      '/proc/%s/fd/3' % pid: 'socket:[77777777]',
      '/proc/%s/fd/4' % pid: 'pipe:[30303]',
      '/proc/%s/fd/5' % pid: 'pipe:[40404]',
    }[param]

    path_exists_mock.side_effect = lambda param: {
      '/proc/net/tcp6': False,
      '/proc/net/udp6': False
    }[param]

    open_mock.side_effect = lambda param, mode: {
      '/proc/net/tcp': io.BytesIO(SL_WIDTH_CHANGE),
      '/proc/net/udp': io.BytesIO(TITLE_LINE)
    }[param]

    expected = [
      Connection('17.17.17.17', 4369, '51.34.34.34', 8772, 'tcp', False),
      Connection('34.34.34.34', 8738, '68.51.51.51', 13141, 'tcp', False),
      Connection('51.51.51.51', 13107, '85.68.68.68', 17510, 'tcp', False),
    ]

    self.assertEqual(expected, proc.connections(pid))



import re

from docutils.utils import unescape
from docutils.nodes import reference
from docutils.parsers.rst.roles import set_classes


def role_trac(name, rawtext, text, lineno, inliner, options={}, content=[]):
  """
  Aliases :trac:`1234` to 'https://trac.torproject.org/1234'.

  :param name: the role name used in the document
  :param rawtext: the entire markup snippet, with role
  :param text: the text marked with the role
  :param lineno: the line number where rawtext appears in the input
  :param inliner: the inliner instance that called us
  :param options: directive options for customization
  :param content: the directive content for customization
  """

  # checking if the number is valid
  try:
    ticket_num = int(text)

    if ticket_num <= 0:
      raise ValueError
  except ValueError:
    msg = inliner.reporter.error('Invalid trac ticket: %s' % text, line=lineno)
    prb = inliner.problematic(rawtext, rawtext, msg)

    return ([prb], [msg])

  app = inliner.document.settings.env.app
  link_text = 'ticket %s' % unescape(str(ticket_num))

  return (
    [make_link_node(rawtext, app, 'trac_url', link_text, str(ticket_num), options)],
    [],
  )


def role_spec(name, rawtext, text, lineno, inliner, options={}, content=[]):
  """
  Aliases :spec:`25b0d43` to 'https://gitweb.torproject.org/torspec.git/commit/?id=25b0d43'.
  """

  # checking if the input is a valid short commit id

  if not re.match('^[0-9a-f]{7}$', text):
    msg = inliner.reporter.error('Spec tag expects a short commit id (seven hex characters): %s' % text, line=lineno)
    prb = inliner.problematic(rawtext, rawtext, msg)

    return ([prb], [msg])

  app = inliner.document.settings.env.app

  return (
    [make_link_node(rawtext, app, 'spec_url', 'spec', text, options)],
    [],
  )


def make_link_node(rawtext, app, url_type, link_text, slug, options):
  """
  Creates a link to a trac ticket.

  :param rawtext: text being replaced with link node
  :param app: sphinx application context
  :param url_type: base for our url
  :param link_text: text for the link
  :param slug: ID of the thing to link to
  :param options: options dictionary passed to role func
  """

  base_url = getattr(app.config, url_type, None)

  if not base_url:
    raise ValueError("'%s' isn't set in our config" % url_type)

  ref = base_url.format(slug = slug)
  set_classes(options)

  return reference(rawtext, link_text, refuri = ref, **options)


def setup(app):
  """
  Installs the plugin.

  :param app: sphinx application context
  """

  app.add_role('trac', role_trac)
  app.add_config_value('trac_url', None, 'env')

  app.add_role('spec', role_spec)
  app.add_config_value('spec_url', None, 'env')
